pro aslice, dir=dir, nb=nb,level=level,coordperp=coordperp,$
dim=dim,radix=radix,component=component,quiet=quiet,$
velocity=velocity,$
blank=blank,field=field,mesh=mesh,help=help,_extra=e
common HD_param, str_par
;+
; NAME:
;       ASLICE
;
; PURPOSE:
;
;       Extracts a 2D slice out of an HD array, at a given level.
;       That slice is affected to a 2D array of reals.
;
; CATEGORY:
;       Array manipulation routine
;
; CALLING SEQUENCE:
;
;       ASLICE [,DIR=dir] [,NB=nb] [,LEVEL=level] [,COORDPERP=coordperp]
;       [,DIM=dim] [,/HELP] [,RADIX=radix] [,COMPONENT=component]
;       [,BLANK=blank] [,FIELD=field] [,MESH=mesh] [,/VELOCITY]
;
; DIR=dir: the directory where to look for the files. Defaults to '.',
; and is sticky.
;
; NB=nb: the output number of the data that must be used. Defaults to
; 0, and is sticky.
;
; LEVEL=level: The level (integer number) for which one wants to
; extract a slice. Defaults to all 0. It cannot be a vector.
;
; COORDPERP=coordperp: the value of the coordinate perpendicular to
; the cut at which one wants to have a slice. Defauts to 0, i.e. if DIM
; is set to its default (see below) the slice is a (X,Y) slice of the
; system at Z=0, unless the coordinate system is spherical and DIM is
; set to 2 (the default), in which case it defaults to !PI/2.-1d-5,
; corresponding to an equatorial cut.
;
; DIM=dim: 'dim' has to be an integer between 0 and 2, and stands 
; for the dimension perpendicular to the slice. It defaults to 2,
; therefore the slice is in the (X,Y) plane for a cartesian grid
; with no coordinates reordering.
;
; /HELP: prints this help.
;
; RADIX=radix: the radix of the file names to be read. Defaults to
; 'density'.
;
; COMPONENT=component: if the radix is 'velocity', specify the
; component number to be read. Defaults to zero.
;
; BLANK=blank: the value to be attributed to the cells that do not
; intercept the level LEVEL. Defaults to zero.
;
; FIELD=field: the slice is returned in the 2D array of doubles  FIELD
;
; MESH=mesh: the mesh structure that is returned contains more
; information than the simple 2D array field. It not only contains the
; entry MESH.FIELD, but also MESH.NCMIN and MESH.NCMAX, that give the
; integer min and max corner position on the absolute grid (see code
; specifications), and the entries MESH.XRANGE and MESH.YRANGE that
; give the physical range of the grid corners.
;
; /VELOCITY: shortcut for "RADIX='velocity'"
;
;-
if keyword_set(help) OR NOT(Arg_present(field) OR Arg_present(mesh)) then begin
    doc_library, 'aslice'
    return
endif
readstruct,dir=dir,nb=nb
if n_elements(component) EQ 0 then component = 0
if n_elements(radix) EQ 0 then radix = "density"
if keyword_set(velocity) then radix="velocity"
if n_elements(level) EQ 0 then level=0
if n_elements(dim) EQ 0 then dim=2
if n_elements(coordperp) EQ 0 then begin
    coordperp = 0.0
    p=str_par.sdir.in
    if p.coordtype EQ "Spherical" AND dim EQ 2 then begin
        coordperp = !PI/2.-1d-5
    endif
endif
if n_elements(blank) EQ 0 then blank=0.0
dim1=(dim EQ 0)
dim2=2-(dim EQ 2)
;
;
;first scan
;
;
g=str_par.sgrid
nc_minx=10000000L
nc_maxx=-nc_minx
nc_miny=nc_minx
nc_maxy=nc_maxx
desclist,level=level,nb=nb,list=list,dir=dir
if (list[0] NE -1) then begin
    ncpugrids = (size(list))(1)
    for i=0, ncpugrids-1 do begin
        read_field,field=f,edge1=x,edge2=y,edge3=z,size=s,$
                   cpugrid=list(i),/quiet,radix=radix,component=component
        edges=dblarr(3,max(s)+1)
        edges(0,0:s[0])=x
        edges(1,0:s[1])=y
        edges(2,0:s[2])=z
        cut_field,f,sl,edgeperp=reform(edges(dim,0:s[dim])),$
                  cutdim=dim,cutvalue=coordperp,overlap=ov, /nointerp
        if ov EQ 1 then begin
            index = where(g.cgridnumber EQ list(i))            
            parent=g.parent[index]
            ncmin=g.ncorner_min[parent,*]
            ncmax=g.ncorner_max[parent,*]
            if (ncmin[dim1] LT nc_minx) then nc_minx=ncmin[dim1]
            if (ncmin[dim2] LT nc_miny) then nc_miny=ncmin[dim2]
            if (ncmax[dim1] GT nc_maxx) then nc_maxx=ncmax[dim1]
            if (ncmax[dim2] GT nc_maxy) then nc_maxy=ncmax[dim2]
            size=[g.nx[parent],g.ny[parent],g.nz[parent]]
            sz1=size[dim1]
            sz2=size[dim2]
            dn1=(ncmax[dim1]-ncmin[dim1])/sz1
            dn2=(ncmax[dim2]-ncmin[dim2])/sz2
        endif
    endfor
endif
if (nc_minx EQ 10000000L) then begin
    if NOT(keyword_set(quiet)) then begin
        print, 'I do not intercept anything. Change COORDPERP or LEVEL.'
    endif
    field = -1
    return
endif
m = dblarr((nc_maxx-nc_minx)/dn1,(nc_maxy-nc_miny)/dn2)
m = 0.*m+blank
;
;
;second scan
;
;
xmin=1d20
ymin=1d20
xmax=-1d20
ymax=-1d20
if (list[0] NE -1) then begin
    ncpugrids = (size(list))(1)
    for i=0, ncpugrids-1 do begin
        read_field,field=f,edge1=x,edge2=y,edge3=z,size=s,$
                   cpugrid=list(i),/quiet,radix=radix,component=component
        edges=dblarr(3,max(s)+1)
        edges(0,0:s[0])=x
        edges(1,0:s[1])=y
        edges(2,0:s[2])=z
        cut_field,f,sl,edgeperp=reform(edges(dim,0:s[dim])),$
                  cutdim=dim,cutvalue=coordperp,overlap=ov, /nointerp
        if ov EQ 1 then begin
            if min(x) LT xmin then xmin=min(x)
            if min(y) LT ymin then ymin=min(y)
            if max(x) GT xmax then xmax=max(x)
            if max(y) GT ymax then ymax=max(y)
            sz1=(size(sl))[1]
            sz2=(size(sl))[2]
            index = where(g.cgridnumber EQ list(i))            
            cpu=g.cpurank[index]
            if NOT(keyword_set(quiet)) then print, "Collecting CPU #",cpu
            parent = g.parent[index]
            ncmin=g.ncorner_min[parent,*]
            ncmax=g.ncorner_max[parent,*]
            pcmin=g.pcorner_min[index,*]
            pcmax=g.pcorner_max[index,*]
            cx=(ncmin[dim1]-nc_minx)/dn1+pcmin[dim1]
            cy=(ncmin[dim2]-nc_miny)/dn2+pcmin[dim2]
            m(cx:cx+sz1-1,cy:cy+sz2-1)=sl
        endif
    endfor
endif
field=m
if Arg_present(mesh) then begin
    mesh={field:field,ncmin:[ncmin[dim1],ncmin[dim2]],ncmax:[ncmax[dim1],ncmax[dim2]],xrange:[xmin,xmax],yrange:[ymin,ymax]}
endif
return
end
