pro cut_field, input, output, EDGEPERP=e, EDGEPARA1=e1, EDGEPARA2=e2,$
    cutdim=cutdim, cutvalue=cutvalue, overlap=overlap, boundary=bounds,$
    nointerp=nointerp
;+
; NAME:
;       CUT_FIELD
;
; PURPOSE:
;
;       Cut a 2D slice out of a 3D array
;
; CATEGORY:
;       Array manipulation routine
;
; CALLING SEQUENCE:
;
;       CUT_FIELD, input, output, EDGEPERP=e, CUTDIM=dim,
;       CUTVALUE=cutvalue [,OVERLAP = overlap] [,EDGEPARA1=e1, EDGEPARA2=e2,
;       BOUNDARY=bound]
;
; 
; INPUT: the 3D input array
;
; OUTPUT: upon return, contains the 2D slice
;
; EDGEPERP=e: e is a 1D array containing the coordinate value of the
; zone edges perpendicular to the slice (e.g., if you want a
; horizontal slice say in cylindrical coordinates, 'e' needs to
; be the array of the 'z' values of the zones'interfaces).
;
; EDGEPARA1=e1 and EDGEPARA2=e2: same as previous but for the two dimensions
; parallel to the slice. Needed only if a BOUNDARY argument is present.
;
; CUTDIM=dim: dim is the number of the dimension perpendicular to
; which the cut is performed. It is an integer between 0 and 2.
; Note that is the input array size is N1xN2xN3, and if dim=0, then
; the edge array 'e' needs to have a length N1+1. Similarly, if dim=1,
; its length must be N2+1, etc. As a consequence, it is usefull to
; call either READ_FIELD or READ_DESC before a call to CUT_FIELD.
; See the corresponding help.
;
; CUTVALUE=cutvalue: cutvalue is the physical value of the coordinate
; perpendicular to which the 3D array is cut.
;
; OVERLAP=overlap: upon return, overlap contains 1 if a slice has been
; created (if the cutvalue is beyond the extreme values of the mesh
; coordinates, the slice does not intercept the input array, and
; overlap contains 0). It contains -1 if the 'edge' array is mangled
; (i.e. not monotonically increasing).
;
; BOUNDARY=bound: upon return bound is a 2xN array which contains the
; physical coordinates of the slice boundary points, which can later
; be used in a plotting routine. The argument EDGEPARA1 and EDGEPARA2 are
; needed, otherwise an error message is issued.
;-
if n_elements(cutdim) EQ 0 then begin
    doc_library, 'cut_field'
    return
endif
n=(size(input))(cutdim+1)
if (((size(e))(1)) NE n+1) then print, 'Size mismatch'
if ((cutvalue LT min(e)) OR (cutvalue GT max(e))) then begin
    overlap = 0
    output = -1
    return   ; no slice can be cut at this value.
endif
i=0
while (e(i+1) LT cutvalue) do begin
    i=i+1
    if (e(i) LE e(i-1)) then begin
        print, "the 'edge' array is not monotonically increasing!"
        overlap=-1
        return
    end
endwhile
if Arg_present (bounds) then begin
    if (n_elements(e1) EQ 0) OR (n_elements(e2) EQ 0) then begin
        print, 'Error ! I need the EDGEPARA arguments to return the boundaries.'
        return
    endif
    bounds=[[e1(0),e2(0)]]
    n1=(size(e1))(1)-1
    n2=(size(e2))(1)-1
    for z=1,n1 do bounds=[bounds,[e1(z),e2(0)]]
    for z=1,n2 do bounds=[bounds,[e1(n1),e2(z)]]
    for z=n1,0,-1 do bounds=[bounds,[e1(z),e2(n2)]]
    for z=n2,0,-1 do bounds=[bounds,[e1(0),e2(z)]]
endif
overlap=1
if (e(i+1) EQ e(i)) then begin
    ifrac = 0
endif else begin
    ifrac=(cutvalue-e(i))/(e(i+1)-e(i))+i
endelse
nx = (size(input))(1)
ny = (size(input))(2)
nz = (size(input))(3)
xindex=dindgen(nx)
yindex=dindgen(ny)
zindex=dindgen(nz)
if keyword_set (NOINTERP) then begin
	if (cutvalue EQ max(e)) then ifrac=ifrac-0.0001
    case cutdim of
        0: output=reform(input[ifrac, yindex, zindex],ny,nz)
        1: output=reform(input[xindex, ifrac, zindex],nx,nz)
        2: output=reform(input[xindex, yindex, ifrac],nx,ny)
    endcase
endif else begin
    ifrac=ifrac-0.5
    case cutdim of
        0: output=reform(interpolate (input, [ifrac], yindex, zindex, /grid),ny,nz)
        1: output=reform(interpolate (input, xindex, [ifrac], zindex, /grid),nx,nz)
        2: output=reform(interpolate (input, xindex, yindex, [ifrac], /grid),nx,ny)
    endcase
endelse
;print, 'Interception occured on dim ',cutdim, ' for index = ',ifrac
return
end
