pro displaymesh, mesh, xrange=xrange, yrange=yrange, min=min, max=max,$
 auto=auto,box=box,func=func,grid=grid,_extra=e
;+
; NAME:
;       DISPLAYMESH
;
; PURPOSE:
;       Plot on an X device only a mesh structure. A mesh is a
;       structure that consists of a real 2D array plus coordinate
;       information in the form of 1D arrays that contain the
;       interface coordinate between two neighboring zones. The MESH
;       anonymous structure thus has to contain the following entries:
;       MESH.FIELD, that is a 2D real array (float or double) of size
;       MxN.
;       MESH.XGRID, that is a 1D real array of size M+1
;       MESH.YGRID, that is a 1D real array of size N+1
;       This structure can also contain any other additional field,
;       it will simply not be used by the DISPLAYMESH routine.
;
; CATEGORY:
;       Plotting routine
;
; CALLING SEQUENCE:
;       DISPLAYMESH, MESH [,XRANGE=xrange] [,YRANGE=yrange] [,MIN=min]
;       [,MAX=max] [,/AUTO] [,/BOX] [,FUNC=func] [,/GRID]
;
; MESH: the mesh structure to be displayed. If not present or
; undefined, print this help
;
; XRANGE=xrange: if present that field specifies the range in X-value
; over which the plot has to be done, much in the same way as with the
; PLOT routine from IDL. This allows to display a close-up of a part
; of an array.
;
; YRANGE=yrange: idem for the Y axis.
;
; MIN=min and MAX=max: specifies the minimun and maximum value of the
; mesh field to be displayed. By default the routine construct a
; linear color palette between the absolute minimum and maximum of the
; mesh field.
; 
; /AUTO: determine automatically the value of XRANGE and YRANGE that
; correspond to the MESH. Note that if neither /AUTO, XRANGE nor
; YRANGE is present, the coordinate system that has been used for the
; previous plot is used again.
;
; /BOX: display a box around the plotting area, much in the same way
; as the PLOT routine from IDL.
;
; FUNC=func: if this keyword is present, 'func' must be a string that
; corresponds to the name of an existing IDL function, that takes a
; real as a unique  argument, and returns a real, such a FUNC='ALOG' or
; FUNC='SQRT'. This function is applied to the MESH.FIELD prior to the
; display. This circumvents to limitation that a linear color table is
; used to display the array. Note that the MAX and MIN value, in case
; they are specified, do not need to be adapted (the routine
; automatically applies 'func' to the MAX and MIN values).
;
; /GRID: if this keyword is present the grid is materialized (all
; the zone interfaces are drawn).
;
; NOTE:
; ----
; The X device is not erased during the call to DISPLAYMESH. This
; allows one to superimpose successive meshes (this is actually how
; the SLICE procedure works).
;
; The PLOT routines used within DISPLAYMESH (if /BOX or /GRID are
; specified) inherit all extra keywords given to DISPLAYMESH.
;-
help,mesh,/struct,output=test
if ((total(strpos(test,"UNDEFINED"))+(size(test))(1)) NE 0) then begin
   doc_library, 'displaymesh'
    return
endif
fieldpresent = 0
xgridpresent = 0
ygridpresent = 0
if ((total(strpos(test,"FIELD"))+(size(test))(1)) NE 0) then fieldpresent=1
if ((total(strpos(test,"XGRID"))+(size(test))(1)) NE 0) then xgridpresent=1
if ((total(strpos(test,"YGRID"))+(size(test))(1)) NE 0) then ygridpresent=1
if NOT(fieldpresent*xgridpresent*ygridpresent) then begin
    print, "Error: DISPLAYMESH needs the mesh argument to be a structure that"
    print, "contains at least the fields 'field' (2D array), 'xgrid' (vector)"
    print, "and 'ygrid' (vector)"
    return
endif
if n_elements(min) EQ 0 then min=min(mesh.field)
if n_elements(max) EQ 0 then max=max(mesh.field)
nx=(size(mesh.field))[1]
ny=(size(mesh.field))[2]
xmin=min(mesh.xgrid)
xmax=max(mesh.xgrid)
ymin=min(mesh.ygrid)
ymax=max(mesh.ygrid)
if keyword_set(auto) then begin
    !x.range = [xmin,xmax]
    !y.range = [ymin,ymax]
    plot,[0,0],[1,1],/nodata,xstyle=5,ystyle=5,_extra=e
endif
if keyword_set(xrange) then begin
    !x.range = xrange
    plot,[0,0],[1,1],/nodata,xstyle=5,ystyle=5,/noerase,_extra=e
endif
if keyword_set(yrange) then begin
    !y.range = yrange
    plot,[0,0],[1,1],/nodata,xstyle=5,ystyle=5,/noerase,_extra=e
endif
if (xmax LE !x.crange[0]) then return
if (xmin GE !x.crange[1]) then return
if (ymax LE !y.crange[0]) then return
if (ymin GE !y.crange[1]) then return
; We now need to crop the matrix, in order not to saturate the memory
; with the call to the congrid function below, in case the user
; requires a considerable zoom factor (very small x or y range).
; We know that since the above tests did not lead to a return that the
; picture does intersect the window plot.
j = nx-1
while (NOT(mesh.xgrid[j-1] LE !x.crange[1])) AND (j GT 2) do j=j-1
xgrid=mesh.xgrid[0:j+1]
field=mesh.field[0:j,*]
j = 0
while (NOT(mesh.xgrid[j+1] GE !x.crange[0])) AND (j LT nx-2) do j=j+1
xgrid=xgrid[j:*] ; Note: xgrid and not mesh.xgrid as in the above line
field=field[j:*,*] ;Note: field and not mesh.field as in the above line
j = 0
j = ny-1
while (NOT(mesh.ygrid[j-1] LE !y.crange[1])) AND (j GT 2) do j=j-1
ygrid=mesh.ygrid[0:j+1]
field=field[*,0:j]
j=0
while (NOT(mesh.ygrid[j+1] GE !y.crange[0])) AND (j LT ny-2) do j=j+1
ygrid=ygrid[j:*]
field=field[*,j:*] 
xmin=min(xgrid)
xmax=max(xgrid)
ymin=min(ygrid)
ymax=max(ygrid)

; Some trick...
; if field has 1 dimension instead of 2, it means that it had a
; *trailing* size of 1, which we need to reconstruct. This is because
; the above affectation feels allowed to remove any trailing shallow
; dimension... 
sf=(size(field))(0)
s1=(size(field))(1)
if sf EQ 1 then field=reform(field,s1,1)
; End of some trick...
nx = (size(xgrid))(1)-1
dx = (xmax-xmin)/nx
xdata_to_pix = !x.s[1] * !d.x_size
cell_x_pix = dx*xdata_to_pix
ny = (size(ygrid))(1)-1
dy = (ymax-ymin)/ny
ydata_to_pix = !y.s[1] * !d.y_size
cell_y_pix = dy*ydata_to_pix
; The size of the mesh in pixels, on the screen, should
; be cell_x_pix*nx BY cell_y_pix*ny
; However, in order for the size to be *exactly* in agreement
; with the position of the neighbors (in order to avoid either
; a slight overlap or a missing row or column of representating
; pixels between two neighboring CPUs) as imposed by the 'TV'
; procedure below, we use the 'CONVERT_COORD' function as follows:
xposl = (long(convert_coord(xmin,ymin,/data,/to_device)))[0]
yposl = (long(convert_coord(xmin,ymin,/data,/to_device)))[1]
xposr = (long(convert_coord(xmax,ymax,/data,/to_device)))[0]
yposr = (long(convert_coord(xmax,ymax,/data,/to_device)))[1]
if (xposr EQ xposl) then xposr=xposl+1
if (yposr EQ yposl) then yposr=yposl+1
m=congrid(field, xposr-xposl, yposr-yposl)
if (size(m))(0) EQ 1 then begin
    szm = (size(m))(1)
    m=reform(m,szm,1)
endif
left_trunc_x = 0
if (!x.crange[0] GT xmin) then begin
    left_trunc_x = long((!x.crange[0]-xmin)*xdata_to_pix+.5)
endif
right_trunc_x = 0
if (!x.crange[1] LT xmax) then begin
    right_trunc_x = long((xmax-!x.crange[1])*xdata_to_pix+.5)
endif
left_trunc_y = 0
if (!y.crange[0] GT ymin) then begin
    left_trunc_y = long((!y.crange[0]-ymin)*ydata_to_pix+.5)
endif
right_trunc_y = 0
if (!y.crange[1] LT ymax) then begin
    right_trunc_y = long((ymax-!y.crange[1])*ydata_to_pix+.5)
endif
mx=(size(m))(1)
my=(size(m))(2)
if (left_trunc_x GE mx) then return
if (left_trunc_y GE my) then return
if (right_trunc_x GE mx) then return
if (right_trunc_y GE my) then return
m=m(left_trunc_x:mx-1-right_trunc_x,left_trunc_y:my-1-right_trunc_y)
vmin=min
vmax=max
if keyword_set(func) then begin
   m=call_function(func, m)
   vmin=call_function(func, min)
   vmax=call_function(func, max)
endif
m=bytscl(m,min=vmin,max=vmax)
xpos = max([xmin,!x.crange[0]])
ypos =  max([ymin,!y.crange[0]])
xposl = (long(convert_coord(xpos,ypos,/data,/to_device)))[0]
yposl = (long(convert_coord(xpos,ypos,/data,/to_device)))[1]
tv, m, xposl, yposl, /device
if keyword_set(box) then begin
    !x.range=!x.crange
    !y.range=!y.crange
    plot,[0,0],[1,1],/nodata,/xstyle,/ystyle,/noerase,_extra=e
endif
if keyword_set(grid) then begin
    for i=0,nx do begin
        plots,xgrid(i),ymin,/data
        plots,xgrid(i),ymax,/data,_extra=e,/continue,noclip=0
    endfor
    for i=0,ny do begin
        plots,xmin,ygrid(i),/data
        plots,xmax,ygrid(i),/data,_extra=e,/continue,noclip=0
    endfor
endif
end
