pro equatorial_vel, colatitude=colatitude, nb=nb, genmin=genmin,$
 genmax=genmax, path=path, verbose=verbose, _extra=e
;+
; NAME:
;       EQUATORIAL_VEL
;
; PURPOSE:
;
;       Plot an equatorial 2D slice out of a scalar quantity on a
;       spherical mesh or a set of nested spherical meshes
;
; CATEGORY:
;       Plotting routine
;
; CALLING SEQUENCE:
;
;       EQUATORIAL_VEL, COLATITUDE=c, NB=nb [,GENMIN=gmin]
;       [,GENMAX=gmax] [,PATH=path] [,/VERBOSE]
;
; NOTE: in addition to these keywords, EQUATORIAL_VEL inherits all the
; keywords specific to the PLOT2D_VEL routine (see its helpfile).
; 
; COLATITUDE=c: the colatitude at which the meridional slice must be
; cut. Note that the name of the routine is therefore misleading: the
; cut needs not be equatorial (colatitude = !PI/2). 
;
; NB=nb: the output number of the data that must be used.
;
; GENMIN=gmin: only meshes with generation larger than 'gmin' are
; taken into account.
;
; GENMAX=gmax: only meshes with generation smaller than 'gmax' are
; taken into account.
;
; /VERBOSE: verbose output. Tells which descriptors are intercepted,
; where, and the min and max value of the velocity module.
;
;-
if (n_elements(nb) EQ 0) OR (n_elements(colatitude) EQ 0) then begin
    doc_library, 'equatorial_vel'
    return
endif
if keyword_set (path) then path=path else path='.'
if (n_elements(genmin) EQ 0) then genmin = 0
if (n_elements(component) EQ 0) then component = 0
if (n_elements(genmax) EQ 0) then genmax = genmin+8
desclist, generation=indgen(genmax-genmin+1)+genmin, nb=nb, list=list, path=path
list=[list]
vr_list=[0.0]
va_list=[0.0]
xx=[0.0]
yy=[0.0]
rr=[0.0]
if (list(0) NE -1) then begin
    for k=0,(size(list))(1)-1 do begin
        read_field,radix='velocity', nb=nb, desc=list(k),field=vrad,path=path,$
                   /quiet, edge3=col, component=1
        read_field,radix='velocity', nb=nb, desc=list(k),field=vazim,path=path,$
                   /quiet, edge3=col, component=0
        read_desc,desc=list(k),path=path,azimuth=az,radius=x, lastlevel=mask, gen=gen
        r_slice = reform(x(*,*,0))
        x_slice = reform(x(*,*,0)*cos(az(*,*,0)))
        z_slice = reform(x(*,*,0)*sin(az(*,*,0)))
        cut_field,vrad,edgeperp=col,cutdim=2,cutvalue=colatitude,overlap=overlap,slice_vrad
        cut_field,vazim,edgeperp=col,cutdim=2,cutvalue=colatitude,overlap=overlap,slice_vazim
        if (overlap EQ 1) then begin
            if keyword_set(verbose) then print, 'Descriptor ', list(k),' intercepted'
            cut_field,mask,edgeperp=col,cutdim=2,cutvalue=colatitude,maskslice
            if (gen LT genmax) then begin
                b = where(maskslice GT 0.5)
                vr_list=[vr_list,slice_vrad(b)]
                va_list=[va_list,slice_vazim(b)]
                xx=[xx,x_slice(b)]
                yy=[yy,z_slice(b)]
                rr=[rr,r_slice(b)]
            endif else begin
                b = where(maskslice*0.0 LT 0.1)
                vr_list=[vr_list,slice_vrad(b)]
                va_list=[va_list,slice_vazim(b)]
                xx=[xx,x_slice(b)]
                yy=[yy,z_slice(b)]
                rr=[rr,r_slice(b)]
            endelse
        endif
    endfor
endif
if ((size(xx))(1) EQ 1) then begin
    print, 'No mesh intercepted'
    return
endif
xx=xx(1:*)
yy=yy(1:*)
rr=rr(1:*)
vr_list=vr_list(1:*)
va_list=va_list(1:*)
vx = vr_list*xx/rr-va_list*yy
vy = vr_list*yy/rr+va_list*xx
if keyword_set(verbose) then print, 'Maximum velocity value = ', max(sqrt(vx*vx+vy*vy))
if keyword_set(verbose) then print, 'Minimum velocity value = ', min(sqrt(vx*vx+vy*vy))
!z.type=0
plot2d_vel,vx,vy,xx,yy,_extra=e
return
end
