; NAME:
;	FMgrMesh
;
; PURPOSE:
;	This object serves as a graphical representation of a mesh,
;	which subclasses from the IDLgrModel class. The mesh can
;       be either cartesian, cylindrical or spherical.
;
; CATEGORY:
;	Object graphics.
;
; CALLING SEQUENCE:
;	To initially create:
;	       	oMesh = OBJ_NEW('FMgrMesh'[,xlim [,ylim [,zlim]]],$
;                       meshtype = {0|1|2}, xstep=xstep, ystep=ystep,$
;                       zstep=zstep, /boundary) 
;
;	To retrieve a property value:
;		oMesh->GetProperty
;
;	To set a property value:
;		oMesh->SetProperty
;
;	To print to the standard output stream the current properties of 
;	the mesh:
;		oMesh->Print[, /full]
;
;	To destroy:
;		OBJ_DESTROY, oMesh
;
; KEYWORD PARAMETERS:
;   FMGRMESH::INIT:
;	<Note that keywords accepted by IDLgrModel::Init and/or
;	 IDLgrPolyline::Init are also accepted here.>
;       XLIM, YLIM and ZLIM: coordinates of zones interfaces. They are
;       floating arrays. If omitted they default respectively to
;       [0.0,1.0] for xlim and ylim, and [0.0] for zlim. If we call
;       nx, ny and nz the respective sizes of these arrays, the
;       resulting mesh contains (nx-1)*(ny-1)*(nz-1) cells. If one or
;       several variables of the set {nx, ny, nz} then the
;       corresponding mesh is either a surface or a curved line.
;       XSTEP, YSTEP and ZSTEP: each respectively 1st, 2nd or third
;       coordinate isosurface is plotted every X,Y,ZSTEP cells,
;       resulting in a less dense mesh (for visibility
;       purpose). X,Y,ZSTEP does not need be a submultiple of
;       nx,y,z. Even if they are not, the outer boundaries are plotted
;       anyway. X,Y,ZLIM can be dynamically resized during the object
;       lifetime (e.g. mesh->SetProperty, zlim = findgen(40))
;       MESHTYPE: 0 for cartesian (in which case x, y and zlim are
;       self-explanatory), 1 for cartesian (in which case x is the
;       radius, y is the azimuth, and z the height), any other value
;       for spherical (in which case x is the radius, y the azimuth,
;       and z is the latitude).
;       BOUNDARY: set to non-zero value for the mesh to represent only
;       its boundary edges and vertices.
;
;
;       Example
;       Create a horizontal 3x2 cartesian mesh, with boundaries only
;       mesh = obj_new ('FMgrMesh', findgen(3)*.3, findgen(4)*.2, $
;       /boundary)
;       Show inner zones
;       mesh->SetProperty, b=0
;       Show every other y-interface
;       mesh->SetProperty, ystep=2
;       Show every y-interface
;       mesh->SetProperty, ystep=1
;       Replicate the mesh 5 times in z
;       mesh->SetProperty, zlim=findgen(5)*.3
;       The mesh is now cylindrical:
;       mesh->SetProperty, meshtype=1 
;       Display mesh properties:
;       mesh->Print, /full
;       obj_destroy, mesh
;---------------------------------------------------------------------

function FMgrMesh::Init, xlim, ylim, zlim, Meshtype=meshtype,$
                 xstep = xstep, ystep = ystep, zstep = zstep,$
                 boundary = boundary, _EXTRA=e
IF (self->IDLgrModel::Init(_EXTRA=e) NE 1) THEN RETURN, 0
if (n_elements (xlim) EQ 0) then xlim = [0.0,1.0]
if (n_elements (ylim) EQ 0) then ylim = [0.0,1.0]
if (n_elements (zlim) EQ 0) then zlim = 0.0
xlim = [xlim]
ylim = [ylim]
zlim = [zlim]
nx = (size(xlim))(1)
ny = (size(ylim))(1)
nz = (size(zlim))(1)
self.xlim = ptr_new(findgen(nx))
self.ylim = ptr_new(findgen(ny))
self.zlim = ptr_new(findgen(nz))
*self.xlim = xlim
*self.ylim = ylim
*self.zlim = zlim
xst = 1
yst = 1
zst = 1
if (n_elements (xstep) NE 0) then xst = max([xstep,1])
if (n_elements (ystep) NE 0) then yst = max([ystep,1])
if (n_elements (zstep) NE 0) then zst = max([zstep,1])
self.xstep = xst
self.ystep = yst
self.zstep = zst
bdr = 0
if (n_elements (boundary) NE 0) then begin
    if (boundary NE 0) then bdr = 1 
endif
M_T = 0
if (N_ELEMENTS(Meshtype) NE 0) then M_T = Meshtype
self.Meshtype = M_T
self.boundary = bdr
self.oLine = OBJ_NEW('IDLgrPolyline', _EXTRA=e)
self->Add, self.oLine
self->MeshBuild
return,1
END
;--------------------------------------------------------------------
PRO FMgrMesh::Cleanup
obj_destroy, self.oLine
PTR_FREE, self.xlim
PTR_FREE, self.ylim
PTR_FREE, self.zlim
self->IDLgrModel::Cleanup
return
end
;--------------------------------------------------------------------
PRO FMgrMesh::SetProperty, XLIM=xlim, YLIM=ylim, ZLIM=zlim, $
            XSTEP=xstep, YSTEP=ystep, ZSTEP=zstep,$
            MESHTYPE=meshtype, Boundary=boundary, _EXTRA=e
self->IDLgrModel::SetProperty, _EXTRA=e
self.oLine->SetProperty, _EXTRA=e
IF (N_ELEMENTS(xstep) NE 0) THEN $
  self.xstep = max([xstep,1])
IF (N_ELEMENTS(ystep) NE 0) THEN $
  self.ystep = max([ystep,1])
IF (N_ELEMENTS(zstep) NE 0) THEN $
  self.zstep = max([zstep,1])
IF (N_ELEMENTS(meshtype) NE 0) THEN $
  self.meshtype = meshtype
IF (N_ELEMENTS(boundary) NE 0) THEN $
  self.boundary = boundary
IF (N_ELEMENTS(xlim) NE 0) THEN BEGIN
    xlim = [xlim]
    nx = (size(xlim))(1)
    PTR_FREE, self.xlim
    self.xlim = ptr_new(findgen(nx))
    *self.xlim = xlim
ENDIF
IF (N_ELEMENTS(ylim) NE 0) THEN BEGIN
    ylim = [ylim]
    ny = (size(ylim))(1)
    PTR_FREE, self.ylim
    self.ylim = ptr_new(findgen(ny))
    *self.ylim = ylim
ENDIF
IF (N_ELEMENTS(zlim) NE 0) THEN BEGIN
    zlim = [zlim]
    nz = (size(zlim))(1)
    PTR_FREE, self.zlim
    self.zlim = ptr_new(findgen(nz))
    *self.zlim = zlim
ENDIF
self->MeshBuild
RETURN
END
;--------------------------------------------------------------------
PRO FMgrMesh::Print, full=full
if (N_ELEMENTS(full) NE 0) then begin
    print, 'xlim = ', *self.xlim
    print, 'ylim = ', *self.ylim
    print, 'zlim = ', *self.zlim
endif
print, 'xstep = ', self.xstep
print, 'ystep = ', self.ystep
print, 'zstep = ', self.zstep
if (self.boundary NE 0) then print, 'Boundaries only'
case self.meshtype of
    0: print, 'Cartesian mesh'
    1: print, 'Cylindrical mesh'
    else: print, 'Spherical mesh'
endcase
return
end
;--------------------------------------------------------------------
PRO FMgrMesh::GetProperty, XLIM=xlim, YLIM=ylim, ZLIM=zlim, $
            XSTEP=xstep, YSTEP=ystep, ZSTEP=zstep,$
            MESHTYPE=meshtype, Boundary=boundary, _REF_EXTRA=re
self->IDLgrModel::GetProperty, _EXTRA=re
self.oLine->GetProperty, _EXTRA=re
xlim = *self.xlim
ylim = *self.ylim
zlim = *self.zlim
xstep= self.xstep
ystep= self.ystep
zstep= self.zstep
meshtype = self.meshtype
boundary = self.boundary
return
END
;--------------------------------------------------------------------
PRO FMgrMesh::MeshBuild
x = *self.xlim
y = *self.ylim
z = *self.zlim
nx = (size(x))(1)
ny = (size(y))(1)
nz = (size(z))(1)
xx = dblarr(nx*ny*nz)
yy = xx
zz = xx
xst = self.xstep
yst = self.ystep
zst = self.zstep
if (self.boundary NE 0) then begin
    xst = max([nx-1,1])
    yst = max([ny-1,1])
    zst = max([nz-1,1])
endif
vertex = dblarr(3,nx*ny*nz)
l = 0L
M_T = self.MeshType
;for i=0, nx-1 do begin
;    for j=0, ny-1 do begin
;        for k = 0, nz-1 do begin
;            case M_T of
;                0: begin
;                    xx(l) = x(i)
;                    yy(l) = y(j)
;                    zz(l) = z(k)
;                end
;                1: begin
;                    xx(l) = x(i)*cos(y(j))
;                    yy(l) = x(i)*sin(y(j))
;                    zz(l) = z(k)
;                end
;                else: begin
;                    xx(l) = x(i)*cos(y(j))*sin(z(k))
;                    yy(l) = x(i)*sin(y(j))*sin(z(k))
;                    zz(l) = x(i)*cos(z(k))
;                end
;            endcase
;            l = l+1
;        endfor
;    endfor
;endfor
case M_T of
    0: begin
        xx = reform(transpose(rebin(x,nx,ny*nz)),nx*ny*nz)
        yy = reform(rebin(reform(transpose(rebin(y,ny,nz)),ny*nz),ny*nz,nx),nx*ny*nz)
        zz = reform(rebin(z,nz,nx*ny),nx*ny*nz)
    end
    1: begin
        xx = reform(transpose(rebin(reform(x##cos(y),nx*ny),nx*ny,nz)),nx*ny*nz)
        yy = reform(transpose(rebin(reform(x##sin(y),nx*ny),nx*ny,nz)),nx*ny*nz)
        zz = reform(rebin(z,nz,nx*ny),nx*ny*nz)
    end
    2 : begin
        xx = reform(reform(x##cos(y),nx*ny)##sin(z),nx*ny*nz)
        yy = reform(reform(x##sin(y),nx*ny)##sin(z),nx*ny*nz)
        zz = reform(reform(x##(y*0.0+1.0),nx*ny)##cos(z),nx*ny*nz)
    end
endcase
i = -xst
while (i LT nx-1) do begin 
    i = i + xst
    if (i GE nx) then i = nx-1
    j = -yst
    while (j LT ny-1) do begin
        j = j + yst
        if (j GE ny) then j = ny-1
        if (n_elements (ndata) EQ 0) then $
          ndata = [nz, lindgen(nz)+i*ny*nz+j*nz] $
        else $
          ndata = [ndata, nz, lindgen(nz)+i*ny*nz+j*nz]
    endwhile
endwhile
i = -xst
while (i LT nx-1) do begin 
    i = i + xst
    if (i GE nx) then i = nx-1
    k = -zst
    while (k LT nz-1) do begin
        k = k + zst
        if (k GE nz) then k = nz-1
        ndata = [ndata, ny, lindgen(ny)*nz+i*ny*nz+k]
    endwhile
endwhile
j = -yst
while (j LT ny-1) do begin
    j = j + yst
    if (j GE ny) then j = ny-1
    k = -zst
    while (k LT nz-1) do begin
        k = k + zst
        if (k GE nz) then k = nz-1
        ndata = [ndata, nx, lindgen(nx)*nz*ny+j*nz+k]
    endwhile
endwhile
ndata = [ndata,-1]
vertex(0,*) = xx
vertex(1,*) = yy
vertex(2,*) = zz
self.oLine->SetProperty, data=vertex, polylines=ndata
xx = 0
yy = 0
zz = 0
return
end
;--------------------------------------------------------------------
PRO FMgrMesh__define
compile_opt idl2, hidden

struct_hide, {FMgrMesh, inherits IDLgrModel, $
              xlim: PTR_NEW(), ylim: PTR_NEW(), zlim: PTR_NEW(),$
              MeshType:0L,$
              xstep:1L, ystep:1L, zstep:1L, boundary:0L,$
              oLine: OBJ_NEW()}
end
