pro getdesc, grid=grid, nb=nb, levels=levels, dir=dir,$
cpus=cpus, force=force, all=all, add=add, _extra=e
common HD_param, str_par
;+
; NAME:
;       GETDESC
;
; PURPOSE: 
;       Read information about mesh structure from a
;       Descriptor[i].dat file, and construct an IDL graphics
;       object, that can be visualized e.g. with XOBJVIEW.
;
; CATEGORY:
;       IDL object creation routine.
;
; CALLING SEQUENCE: 
;       GETDESC, GRID=grid [, NB=nb] [, LEVELS=levels]
;       [, DIR=directory] [, CPUS=cpus] [, /FORCE] [, /ALL]
;
; The GRID argument is mandatory. The set of mesh objects is returned
; in the grid variable.
;
; NB=nb: Read mesh information in 'Descriptor[nb].dat'
; file. Defaults to 0.
;
; LEVELS=levels: levels is a vector that is a list of levels to be
; displayed. If not specified, all levels are displayed. Allows to
; display non-contiguous levels: LEVELS=[0,2] displays only levels 0
; and 2. In order to display levels from 0 to 2 one needs to give:
; LEVELS=[0,1,2]. 
;
; DIR=directory: specifies the directory where to look for
; 'Descriptor[nb].dat'. Defaults to '.'
;
; /FORCE: if the variable given to the GRID keyword already contains
; an allocated object, force its destruction before building a
; mesh. This avoids memory leakage. If the /FORCE keyword is not
; specified while GRID already contains an object, the routine issues
; an error message and exits.
;
; /ADD: if the variable given to the GRID keyword already contains an
; allocated object, add the new one to the already existing one. It is
; an error to specify both /FORCE and /ADD.
;
; CPUS=cpus: specify whether to show how the grids are split between
; different cpus, and which CPUs are displayed. If 'cpus' is a scalar,
; then all CPUs are displayed (equivalent to /CPUS). If 'cpus' is a
; vector, then every CPU that belongs to the vector list is
; displayed. Beware that if you only want to see the CPU number 3, you
; need to typeset: CPUS=[3], rather than CPUS=3, since in this last
; case all CPUs would be displayed. If the CPUS keyword is not
; specified, than only the parent grids are displayed, and their
; splitting over the CPUs is not shown.
;
; /ALL: show everything, even CPUs or levels that do not belong to the
; lists explicitely provided by the LEVELS or CPUS keyword. If a mesh
; does not belong to these lists, it is displayed with a dotted line
; rather than a solid line.
;
; Example: show CPUs number 2 and 4 of mesh(es) at level 3. Everything
; else is shown in dotted lines:
;
; getdesc, grid=g, cpus=[2,4], lev=3, /all
; xobjview, g
;
;-
if (NOT(ARG_PRESENT(GRID))) then begin
    doc_library, 'getdesc'
    return
endif
help, grid, /object, output=type
type = [type]
if (keyword_set(add) AND NOT((size(type))(1) GT 1)) then begin
    print, "Error: cannot add to anything."
    print, "Grid does not exist yet."
    return
endif
if ((size(type))(1) GT 1) then begin
    if keyword_set(force) then begin
        obj_destroy, grid
    endif else begin
        if (NOT(keyword_set(add))) then begin
            print, "Error: GRID is already an allocated object."
            print, "Use another variable or re-run with the /FORCE"
            print, "flag switched on to allow destruction prior"
            print, "to the mesh object build up."
            return
        endif
    endelse
endif
listcpu=indgen(4096)
if NOT(n_elements(cpus) EQ 0) then listcpu=[cpus]
mesh_color=[[0,0,0],[255,0,0],[0,255,0],[0,0,255],[255,255,0],[255,0,255],[0,255,255]]
if (n_elements(levels) EQ 0) then levels=indgen(1000)-10
levels = [levels]
if (NOT(keyword_set(ADD))) then grid = obj_new('IDLgrModel')
readstruct,dir=dir,nb=nb
s=str_par.sgrid
ngh = s.nghost
ndim = s.ndim
for i=0, s.nb_cpu_grids-1 do begin
    pedges = ptrarr(3)
    coordnb = s.coordnb
    for k=0,2 do pedges[k] = ptr_new(s.edges[s.parent[i],k,*])
    pcorner_min = s.pcorner_min(i,*)
    pcorner_max = s.pcorner_max(i,*)
    level = s.level(s.parent(i))
    if (([where(levels-level EQ 0)])(0) NE -1) then levpresent=1
    if NOT(keyword_set(cpus)) then begin
        pcorner_min = [0,0,0]
        pcorner_max = [s.nx(s.parent(i)),$
                           s.ny(s.parent(i)),$
                           s.nz(s.parent(i))]
    endif
    present = 1
    coordtype=str_par.sgrid.coordtype
    if keyword_set(cpus) then present=0
    if (([where(listcpu-s.cpurank(i) EQ 0)])(0) NE -1) then present=1
    if ((present*levpresent EQ 1) OR keyword_set(all)) then begin
        subgrid = obj_new('FMgrMesh',$
               (*pedges[coordnb[0]])[ngh+pcorner_min[coordnb[0]]:$
                                     ngh+pcorner_max[coordnb[0]]],$
               (*pedges[coordnb[1]])[ngh+pcorner_min[coordnb[1]]:$
                                     ngh+pcorner_max[coordnb[1]]],$
               (*pedges[coordnb[2]])[ngh+pcorner_min[coordnb[2]]:$
                                     ngh+pcorner_max[coordnb[2]]],$
                          meshtype=coordtype,linestyle=1-present*levpresent,/boundary,$
                          color=mesh_color(*,min([level,(size(mesh_color))(2)-1])),$
                          _extra=e)
        grid->Add, subgrid
    endif
endfor
close,1
return
end
