pro integrate, nb=nb, path=path, field=field, aspectratio=aspectratio, thickness=thickness, surfdens=surfdens,$
    noplot=noplot, polarplot=polarplot, shadesurf=shadesurf, fullfield=fullfield, _extra=e, radius=radius, profile=profile
;+
; NAME:
;       INTEGRATE
;
; PURPOSE:
;
;       Compute a 2D field from a vertical (colatitude) integration of
;       the 3D base field on a spherical mesh. The integrated field
;       can be either the surface density, or the disk thickness, or
;       the disk aspect ratio (H/R). Optionally the routine can plot
;       the resulting field.
;
; CATEGORY:
;       Reduction/Plotting routine
;
; CALLING SEQUENCE:
;
;       INTEGRATE, NB=nb [,PATH=path] [,FIELD=field] [,/ASPECTRATIO]
;       [,/THICKNESS] [,/SURFDENS] [,/NOPLOT] [,/POLARPLOT]
;       [,/SHADESURF] {,FULLFIELD=fullfield] [,RADIUS=radius]
;       [,PROFILE=profile]
;
; NB=nb: the output number of the data that must be
; used. Mandatory. If this keyword is not present this help is
; printed.
;
; PATH=path: specifies the path where to look for output.
;
; FIELD=field: upon return the 'field' variable contains the computed
; 2D field.
;
; /ASPECTRATIO: set this keyword to compute the aspectratio
;
; /THICKNESS: set this keyword to compute the disk thickness
;
; /SURFDENS: set this keyword to compute the disk surface density (default)
;
; /NOPLOT: set this keyword to prevent any plotting of the computed
; field. The default is a CONTOUR plot in (azimuth,radius) plane
; (i.e. the disk looks 'rectangular'). Any additional CONTOUR
; procedure keyword can be given to the INTEGRATE procedure.
;
; /POLARPLOT: set this keyword to display the computed 2D field in the
; (x,y) plane (i.e. the disk looks 'circular'). Any additional CONTOUR
; procedure keyword can be given to the INTEGRATE procedure.
;
; /SHADESURF: set this keyword to display the computed 2D field with
; the SHADE_SURF command. Any additional SHADE_SURF procedure keyword
; can be given to the INTEGRATE procedure.
;
; FULLFIELD=fullfield: upon return fullfield contains a structure
; which contains not only the field array (i.e. surface density, or
; aspect ratio, etc.) but also two additional arrays (radius and
; azimuth corresponding to each value of the field).
;
; RADIUS=radius: upon return radius contains a 1D array of the radii
; values. Usefull in conjunction with the PROFILE keyword.
; 
; PROFILE=profile: upon return contains the azimuthally averaged
; values for the field. Usefull in conjunction with the RADIUS keyword.
;-
if (n_elements(nb) EQ 0) then begin
    doc_library, 'integrate'
    return
endif
read_field,radix=radix, nb=nb, desc=0,field=m,path=path,/quiet,center1=az,center2=rad
read_desc,desc=0,path=path,radius=rr_array, colat=col_array, volume=vol, azimuth=az_array, inter3=isurf
field=total(m*vol/isurf,3)
fullfield=create_struct('field',field, 'radius', rr_array(*,*,(size(rr_array))(3)-1), 'azimuth', az_array(*,*,(size(az_array))(3)-1))
if keyword_set(aspectratio) then field=sqrt(total(m*(col_array-!PI/2.)^2.0*vol,3)/total(m*vol,3))
if keyword_set(thickness) then field=sqrt(total(rr_array^2*m*(col_array-!PI/2.)^2.0*vol,3)/total(m*vol,3))
radius = rad
profile = total(field,1)/((size(field))(1))
if keyword_set(noplot) then return
if keyword_set(polarplot) then begin
    x=reform(rr_array(*,*,0)*cos(az_array(*,*,0)))
    y=reform(rr_array(*,*,0)*sin(az_array(*,*,0)))
    contour,field,x,y,/irr,_extra=e
    return
endif
if keyword_set(shadesurf) then begin
    shade_surf, field, az, rad, _extra=e
    return
endif
contour,field,az,rad,_extra=e
return
end
        

        



