pro levelgen, field, levels, number=number, min=min, max=max, linear=linear, exponent=exponent, log=log, verbose=verbose
;+
; NAME:
;       LEVELGEN
;
; PURPOSE:
;
;       Construct a vector of levels for a given input array. These
;       levels can later be used in the CONTOUR procedure. Their
;       spacing, as well as the min and max levels or their number,
;       can be customized.
;
; CATEGORY:
;       Array manipulation routine
;
; CALLING SEQUENCE:
;
;       LEVELGEN, field, levels [,NUMBER=number] [,MIN=min] [,MAX=max]
;       [,/LINEAR] [,EXPONENT=exponent] [,/LOG] [,/VERBOSE]
; 
; field: the array which is used to determine the levels. Its format
; is arbitrary, since only its min and max values are used.
;
; levels: upon return contains a 1D vector that is the sequence of
; levels.
;
; NUMBER=number. Sets the number of levels. Defaults to 60.
;
; MIN=min. Overrides the lower level value, which otherwise is set to
; the minimum of 'field'.
;
; MAX=max. Overrides the larger level value, which otherwise is set to
; the maximum of 'field'.
;
; /LINEAR: the levels are in arithmetic sequence. This is the default.
;
; EXPONENT=exponent: the levels^exponent are in arithmetic
; sequence. For 0<exponent<1 this is useful to enhance low intensity
; regions.
;
; /LOG: the levels are in geometric sequence.
;
; /VERBOSE: prints the minium and maximum value of 'field'
;-
if (n_elements(field) EQ 0) then begin
    doc_library, 'levelgen'
    return
endif
if keyword_set(verbose) then print,'max=',max(field),'  and min=',min(field)
if (n_elements(min) EQ 0) then min=min(field)
if (n_elements(max) EQ 0) then max=max(field)
if (n_elements(number) EQ 0) then number=60
levels=(dindgen(number))/(number-1.0)*(max-min)+min
if (keyword_set (log)) then begin
    levels=exp((dindgen(number))/(number-1.0)*(alog(max/min))+alog(min))
    return
endif
if keyword_set(exponent) then begin
    levels=(dindgen(number))/(number-1.0)*(max^exponent-min^exponent)+min^exponent
    levels=levels^(1./exponent)
endif
return
end
