pro meridian, radix=radix, azimuth=azim, nb=nb, genmin=genmin,$
 genmax=genmax, even=even, odd=odd, path=path,$
 maxvalue=maxvalue,minvalue=minvalue, normalize=normalize, $
 verbose=verbose, component=component, dots=dots, field=field, _extra=e
;+
; NAME:
;       MERIDIAN
;
; PURPOSE:
;
;       Plot a meridional 2D slice out of a scalar quantity on a
;       spherical mesh or a set of nested spherical meshes
;
; CATEGORY:
;       Plotting routine
;
; CALLING SEQUENCE:
;
;       MERIDIAN, AZIMUTH=a, NB=nb [,GENMIN=gmin] [,GENMAX=gmax]
;       [,/EVEN] [,/ODD] [,PATH=path] [,MAXVALUE=max] [,MINVALUE=min]
;       [,/NORMALIZE] [,/VERBOSE] [,COMPONENT=comp] [,RADIX=radix]
;       [,/DOTS] [,FIELD=field]
;
; 
; AZIMUTH=a: the azimuth at which the meridional slice must be cut.
;
; NB=nb: the output number of the data that must be used.
;
; GENMIN=gmin: only meshes with generation larger than 'gmin' are
; taken into account.
;
; GENMAX=gmax: only meshes with generation smaller than 'gmax' are
; taken into account.
;
; /EVEN: the mesh is symmetrized with respect to the equatorial plane,
; assuming the field to be displayed has an even parity in 'z':
; field(-z) = field(z)
;
; /ODD: the mesh is symmetrized with respect to the equatorial plane,
; assuming the field to be displayed has an odd parity in 'z':
; field(-z) = -field(z)
;
; MAXVALUE=max: if this keyword is present the field is chopped and
; any value larger than 'max' is set to max.
;
; MINVALUE=min: if this keyword is present the field is chopped and
; any value smaller than 'min' is set to min.
;
; /NORMALIZE: if this keyword is present the field, before plotting,
; is normalized between 0 and 1 (after the possible chopping described
; above) and displayed with 60 equally spaced levels.
;
; /VERBOSE: verbose output. Tells which descriptors are intercepted,
; where, and the min and max value of the field.
;
; COMPONENT=comp: tells which component of the velocity is to be
; displayed. Needs RADIX='velocity' to work.
;
; RADIX=radix: tells the radix for the file to be read. Defaults to
; 'density'.
;
; /DOTS: display a dot at each zone location. Allows one to appreciate
; the boundaries of finer meshes.
;
; FIELD=field: upon return 'field' contains the sequence of all values
; of the slice.
;-
if (n_elements(nb) EQ 0) OR (n_elements(azim) EQ 0) then begin
    doc_library, 'meridian'
    return
endif
if keyword_set (path) then path=path else path='.'
if (n_elements(genmin) EQ 0) then genmin = 0
if (n_elements(component) EQ 0) then component = 0
if (n_elements(genmax) EQ 0) then genmax = genmin+8
desclist, generation=indgen(genmax-genmin+1)+genmin, nb=nb, list=list, path=path
list=[list]
value=[0.0]
xx=[0.0]
yy=[0.0]
if (list(0) NE -1) then begin
    for k=0,(size(list))(1)-1 do begin
        read_field,radix=radix, nb=nb, desc=list(k),field=m,path=path,$
                   /quiet, edge1=az, component=component, center2=radius,center3=colat
        read_desc,desc=list(k),path=path,z=z,axial=x, lastlevel=mask, gen=gen, size=sz
        if (sz(0) GT 1) then begin
            x_slice = reform(x(0,*,*))
            z_slice = reform(z(0,*,*))
        endif else begin
            x_slice = reform(x)
            z_slice = reform(z)
        endelse
        cut_field,m,edgeperp=az,cutdim=0,cutvalue=azim,overlap=overlap,slice,$
                    edgepara1=radius, edgepara2=colat, boundary=bb
        if (overlap EQ 1) then begin
            if keyword_set(verbose) then print, 'Descriptor ', list(k),' intercepted'
            cut_field,mask,edgeperp=az,cutdim=0,cutvalue=azim,maskslice
            if (gen LT genmax) then begin
                b = where(maskslice GT 0.5)
                value=[value,slice(b)]
                xx=[xx,x_slice(b)]
                yy=[yy,z_slice(b)]
            endif else begin
                b = where(maskslice*0.0 LT 0.1)
                value=[value,slice(b)]
                xx=[xx,x_slice(b)]
                yy=[yy,z_slice(b)]
            endelse
        endif
    endfor
endif
if ((size(xx))(1) EQ 1) then begin
    print, 'No mesh intercepted'
    return
endif
xx=xx(1:*)
yy=yy(1:*)
value=value(1:*)
if keyword_set (even) then begin
    xx=[xx,xx]
    yy=[yy,-yy]
    value=[value,value]
endif
if keyword_set (odd) then begin
    xx=[xx,xx]
    yy=[yy,-yy]
    value=[value,-value]
endif
if keyword_set (maxvalue) then begin
    b=where(value GE maxvalue)
    b=[b]
    if (b(0) NE -1) then value(b)=maxvalue
endif
if keyword_set (minvalue) then begin
    b=where(value LE minvalue)
    b=[b]
    if (b(0) NE -1) then value(b)=minvalue
endif
if keyword_set(verbose) then print, 'Maximum field value = ', max(value)
if keyword_set(verbose) then print, 'Minimum field value = ', min(value)
!z.type=0
if keyword_set (normalize) then begin
    max=max(value)
    min=min(value)
    value=(value-min)/(max-min)
    contour,value,xx,yy,/irr,lev=dindgen(60)/60.0, _extra=e
endif else begin
    contour,(value),xx,yy,/irr,_extra=e
endelse
if keyword_set(dots) then oplot,xx,yy,psym=3
if Arg_present(field) then field=value
return
end
