pro meridian_vel, azimuth=azim, nb=nb, genmin=genmin,$
 genmax=genmax, symmetry=symmetry, path=path,$
 verbose=verbose, _extra=e
;+
; NAME:
;       MERIDIAN_VEL
;
; PURPOSE:
;
;       Plot a meridional 2D velocity field (radial,colatitude) out of
;       a set of 3D spherical meshes.
;
; CATEGORY:
;       Plotting routine
;
; CALLING SEQUENCE:
;
;       MERIDIAN_VEL, AZIMUTH=a, NB=nb [,GENMIN=gmin] [,GENMAX=gmax]
;       [,/SYMMETRY] [,PATH=path] [,/VERBOSE]
;
; NOTE: in addition to these keywords, MERIDIAN_VEL inherits all the
; keywords specific to the PLOT2D_VEL routine (see its helpfile).
; 
; AZIMUTH=a: the azimuth at which the meridional slice must be cut.
;
; NB=nb: the output number of the data that must be used.
;
; GENMIN=gmin: only meshes with generation larger than 'gmin' are
; taken into account.
;
; GENMAX=gmax: only meshes with generation smaller than 'gmax' are
; taken into account.
;
; /SYMMETRY: doubles up the mesh with respect to equatorial plane,
; assuming vrad(-z)=vrad(z) et vz(-z)=-vz(z)
;
; /VERBOSE: verbose output. Tells which descriptors are intercepted,
; where, and the min and max value of the velocity module.
;
;-
if (n_elements(nb) EQ 0) OR (n_elements(azim) EQ 0) then begin
    doc_library, 'meridian_vel'
    return
endif
if keyword_set (path) then path=path else path='.'
if (n_elements(genmin) EQ 0) then genmin = 0
if (n_elements(component) EQ 0) then component = 0
if (n_elements(genmax) EQ 0) then genmax = genmin+8
desclist, generation=indgen(genmax-genmin+1)+genmin, nb=nb, list=list, path=path
list=[list]
vr_list=[0.0]
vc_list=[0.0]
xx=[0.0]
yy=[0.0]
cc=[0.0]
if (list(0) NE -1) then begin
    for k=0,(size(list))(1)-1 do begin
        read_field,radix='velocity', nb=nb, desc=list(k),field=vrad,path=path,$
                   /quiet, edge1=az, component=1, center2=radius,center3=colat
        read_field,radix='velocity', nb=nb, desc=list(k),field=vcol,path=path,$
                   /quiet, edge1=az, component=2, center2=radius,center3=colat
        read_desc,desc=list(k),path=path,z=z,axial=x, lastlevel=mask, gen=gen,colatitude=colatitude, size=sz
        if (sz(0) GT 1) then begin
            x_slice = reform(x(0,*,*))
            z_slice = reform(z(0,*,*))
        endif else begin
            x_slice = reform(x)
            z_slice = reform(z)
        endelse
        col_slice = reform(colatitude(0,*,*))
        cut_field,vrad,edgeperp=az,cutdim=0,cutvalue=azim,overlap=overlap,slice_vrad
        cut_field,vcol,edgeperp=az,cutdim=0,cutvalue=azim,overlap=overlap,slice_vcol
        if (overlap EQ 1) then begin
            if keyword_set(verbose) then print, 'Descriptor ', list(k),' intercepted'
            cut_field,mask,edgeperp=az,cutdim=0,cutvalue=azim,maskslice
            if (gen LT genmax) then begin
                b = where(maskslice GT 0.5)
                vr_list=[vr_list,slice_vrad(b)]
                vc_list=[vc_list,slice_vcol(b)]
                xx=[xx,x_slice(b)]
                yy=[yy,z_slice(b)]
                cc=[cc,col_slice(b)]
            endif else begin
                b = where(maskslice*0.0 LT 0.1)
                vr_list=[vr_list,slice_vrad(b)]
                vc_list=[vc_list,slice_vcol(b)]
                xx=[xx,x_slice(b)]
                yy=[yy,z_slice(b)]
                cc=[cc,col_slice(b)]
            endelse
        endif
    endfor
endif
if ((size(xx))(1) EQ 1) then begin
    print, 'No mesh intercepted'
    return
endif
xx=xx(1:*)
yy=yy(1:*)
cc=cc(1:*)
vr_list=vr_list(1:*)
vc_list=vc_list(1:*)
if keyword_set (symmetry) then begin
    xx=[xx,xx]
    yy=[yy,-yy]
    cc=[cc,!PI-cc]
    vr_list=[vr_list,vr_list]
    vc_list=[vc_list,-vc_list]
endif
vx = vr_list*sin(cc)+vc_list*yy
vy = vr_list*cos(cc)-vc_list*xx
if keyword_set(verbose) then print, 'Maximum velocity value = ', max(sqrt(vx*vx+vy*vy))
if keyword_set(verbose) then print, 'Minimum velocity value = ', min(sqrt(vx*vx+vy*vy))
!z.type=0
plot2d_vel,vx,vy,xx,yy,_extra=e
return
end
