pro plot2d_vel,vx,vy,x,y,raw=raw,dots=dots,length=length,arrow=arrow,diamond=diamond,_extra=e
;+
; NAME:
;       PLOT2D_VEL
;
; PURPOSE:
;       Plots a 2D vector field in cartesian coordinates, even for
;       irregularly spaced data. The display can be either raw, in
;       which case a line or arrow is displayed with its origin at
;       each data point, or displayed in a regular manner after data
;       points triangulation.
;
; CATEGORY:
;       Plotting routine
;
; CALLING SEQUENCE:
;
;       PLOT2D_VEL,vx,vy,x,y [,/RAW] [,/DOTS] [,LENGTH=length]
;       [,/ARROW] [,/DIAMOND]
; 
; vx,vy,x,y: components of the velocity field and position of the
; points where it is defined. They all have to be 1D vectors of the
; same length.
;
; /RAW: displays a line with origin at each (x,y) of input. The
; default length of the longest vector is 1/20th of the plot
; window. This can be overriden with the LENGTH keyword (default
; corresponds to LENGTH=1.0).
;
; /DOTS: displays a dot at each (x,y) of input.
;
; LENGTH=length: redefines the scaling factor for the arrows
; length. Default for raw is explained above, while default for the
; smooth representation, which uses the VELOVECT procedure, is simply
; VELOVECT default (LENGTH=1.0 <==> the longest arrow is one cell
; long). 
;
; /ARROW: in raw mode, this displays arrows instead of lines. Beware
; that in this case there is no clipping possible outside of the
; plotting box.

; /DIAMOND: displays a diamond at each (x,y) of input.
;
;-
if (n_elements(vx) EQ 0) then begin
    doc_library, 'plot2d_vel'
    return
endif
if (n_elements(length) EQ 0) then length=1.0
if keyword_set(raw) then begin
    plot,x,y,/nodata,_extra=e
    nb=(size(x))(1)
    k=(!x.crange[1]-!x.crange[0])/max(sqrt(vx^2+vy^2))*.05*length
    if keyword_set(arrow) then begin
        arrow,x,y,x+vx*k,y+vy*k,/data
    endif else begin
        for i=0,nb-1 do begin
            plots,x(i),y(i),noclip=0,_extra=e
            plots,x(i)+vx(i)*k,y(i)+vy(i)*k,noclip=0,_extra=e,/continue
        endfor
    endelse
endif else begin
    triangulate,x,y,tr
    plot,x,y,/nodata,/xstyle,/ystyle,_extra=e
    xmin=!x.crange[0]
    ymin=!y.crange[0]
    xmax=!x.crange[1]
    ymax=!y.crange[1]
    vxg=trigrid(x,y,vx,tr,/quintic,_extra=e,xgrid=newx,[0.,0.],[xmin,ymin,xmax,ymax],nx=51)
    vyg=trigrid(x,y,vy,tr,/quintic,_extra=e,ygrid=newy,[0.,0.],[xmin,ymin,xmax,ymax],ny=51)
    b = where((vxg EQ 0.0) AND (vyg EQ 0.0))
    b = [b]
    if (b(0) NE -1) then vxg(b)=1e5
    velovect,vxg,vyg,newx,newy,length=length,missing=1e4,_extra=e
endelse
if keyword_set(dots) then oplot,x,y,psym=3
if keyword_set(diamond) then oplot,x,y,psym=4
return
end
    
