pro pmovie, size=size, palette=palette, dirname=dirname,$
procedure=procedure, help=help, nframe=nframe, duration=duration,$
format=format, title=title, signature=signature,$
tduration=tduration, sduration=sduration, ramp=ramp,$
red=red, green=green, blue=blue, color_title=color_title,$
xsize=xsize,ysize=ysize,noreduce=noreduce,offsetnb=offsetnb
;+
; NAME:
;       PMOVIE
;
; PURPOSE:
;       Produces a PNG sequence from a generic display procedure.
;       Optionally displays a title and/or a signature at the begining
;       of the animation. The sequence (that can later be converted
;       to .avi or other format with an adequate command) is produced
;       in a subdirectory of the current working directory.
;
; CATEGORY:
;       Video routine
;
; CALLING SEQUENCE: 
;       PMOVIE, [SIZE=size,] [PALETTE=palette_number,]
;       [DIRNAME=dirname,] [PROCEDURE=procedure,] [NFRAME=nframe,]
;       [TITLE=title,] [TDURATION=title_duration,] [RAMP=ramp,]
;       [RED=red,] [GREEN=green,] [BLUE=blue,] [/NOREDUCE,]
;       [SIGNATURE=signature,] [SDURATION=signature_duration,]
;       [DURATION=duration,] [,/HELP] [,OFFSETNB=offsetnb]
;       [COLOR_TITLE=color_title,] [XSIZE=xsize,] [YSIZE=ysize]
;
; HELP: display this help and exit
;
; SIZE=size: specify the size, in pixels, of the (square) MPEG
; stream. Defaults to 100.
;
; XSIZE=xsize: specify the horizontal size of the MPEG stream. Overides
; any value specified by SIZE.
;
; YSIZE=ysize: specify the vertical size of the MPEG stream. Overides
; any value specified by SIZE.
;
; PALETTE=palette_number: specify the color palette to use. Defaults
; to 3 (RED TEMPERATURE).
;
; DIRNAME=dirname: specify the directory name to use. Defaults to the
; subdirectory 'movie' of the current working directory. If it already
; exists, 'dirname.001', 'dirname.002', etc. are successively tried
; until a directory name that does not already exist is found, and
; a warning is issued.
;
; PROCEDURE=procedure: procedure is a string that is the name of the
; procedure to call to build the succession of frames. It defaults to
; 'templatemov' (see source of templatemov.pro). This procedure has to
; display something to the current display, and it admits one
; parameter, a scalar between 0 and 1. 'procedure, 0' displays the
; first frame, while 'procedure, 1' displays the last frame.
;
; NFRAME=nframe: number of frames to use. Defaults to 24.
;
; DURATION=duration: stream duration in seconds. If this parameter is
; specified, it overrides the NFRAME value with 24*DURATION.
;
; TITLE=title: specify the title that will appear first. It can be
; either a single line string (title='test') or multiline string
; (title = ['This is','a','test'])
;
; TDURATION=title_duration: specify the duration of the title, in
; seconds. This time is not counted in DURATION, so that if one runs:
; IDL> pmovie, title='test', tduration=1.5, duration=2.5
; then the movie will last 4 seconds in total (1.5 sec. of title, then
; 2.5 sec. of animation). The first frame is used as a background of
; the title. Note that if no title is specified, the title phase is
; skipped, regardless of the value of TDURATION. TDURATION defaults to
; 3.0 seconds.
;
; RED=red, GREEN=green, BLUE=blue: specifies the RGB content of the
; color palette to be used (overrides whatever is specified by PALETTE)
; 
; RAMP=ramp: specify the time over which the title brightens, in
; seconds. It defaults to 1.5 seconds. In order to get a maximum
; brightness title instantaneously, use option: ramp=0.0
;
; SIGNATURE=signature: specify a one line string that is used to sign
; the MPEG, like: signature='Author1, 2014'. If there is a title
; phase, then the signature appears at the end of it. Otherwise, the
; signature appears at the begining of the animation.
;
; SDURATION=signature_duration: specify the signature appearance
; duration in seconds. It defaults to 1.0 sec.
; 
; COLOR_TITLE=color_title: color_title is a vector a 3 integers that
; specify the red, green and blue components of any text writtent over
; the image with xyouts (and therefore the title). It defaults to
; [255,255,255]. In the case specifying COLOR_TITLE introduces a
; discontinuity in the color palette, it is not recommended to use a
; ramp to display the title. Rather in that case one wants the title
; to have already the full intensity on the first image.
;
; /NOREDUCE: the color code of each frame is usually reduced from 256
; colors to 254, in order to allocate the COLOR_TITLE color for the movie
; title (see above). For some palettes this is a bad choice, e.g.
; coordinate axes may appear with the wrong color if the palette is
; discontinuous. In that case one can set /NOREDUCE in order to
; avoid this reduction.
;
; OFFSET=offsetnb: png files numbering begins at 'offsetnb'. This leaves
; room for a hand made title, for instance.
;
; SIDE EFFECTS:
;      Uses the Z-buffer. Upon return, sets the plotting device to
;      'X', even if another device was used prior to the call to
;      'pmovie'.
;
; AUTHOR:
;      F. Masset, 23-Dec-2003
;      Modified 21-Jul-2006
;-
if keyword_set(help) then begin
    doc_library, 'pmovie'
    return
endif
com=recall_commands()
comline=com(0)
plotting_proc = 'templatemov'
sizex = 100
sizey = 100
npalette = 3
dir = 'movie'
d_nframe = 24
t_duration = 3
s_duration = 1
d_title=''
d_ramp=1.5
sign=''
signison = 0
titleison = 0
offnb=0L
if keyword_set (size) then begin 
    sizex=size
    sizey=size
endif
if keyword_set (xsize) then sizex = xsize
if keyword_set (offsetnb) then offnb=offsetnb
if keyword_set (ysize) then sizey = ysize
if keyword_set (palette) then npalette=palette
if keyword_set (dirname) then dir=dirname
if keyword_set (procedure) then plotting_proc = procedure
if keyword_set (nframe) then d_nframe = nframe
if keyword_set (tduration) then t_duration = tduration
if keyword_set (sduration) then s_duration = sduration
if (n_elements (ramp) NE 0) then d_ramp = ramp
if keyword_set (duration) then d_nframe = long(duration*24.0+.5)
if keyword_set (signature) then begin 
    sign = signature
    signison = 1
endif
if (file_test(dir)) then begin
    count=1
    dirbase=dir
    repeat begin
        dir=dirbase+strmid('000'+strcompress(string(count),/remove_all),$
                       2,3,/r)
        count=count+1
    endrep until (NOT(file_test(dir)))
    print,'Warning ! The sequence is written in '+dir+' because '+dirbase+' already exists'
endif
file_mkdir,dir
if keyword_set (title) then begin 
    d_title = title
    d_title = [d_title]
    titleison = 1
endif
set_plot,'z'
device, set_resolution=[sizex,sizey]
loadct, npalette
tvlct, cred, cgreen, cblue, /get
if NOT(keyword_set(color_title)) then color_title=[255,255,255]
cred[255]=color_title[0]
cblue[255]=color_title[2]
cgreen[255]=color_title[1]
if NOT(keyword_set(red)) then    red=cred
if NOT(keyword_set(green)) then  green=cgreen
if NOT(keyword_set(blue)) then   blue=cblue
tvlct,red,green,blue
counter=long(offnb)
resolve_routine,plotting_proc
rti=routine_info(plotting_proc,/source)
src=rti(0).path
com='cp '+src+' '+dir+'/'
spawn,com
filename=filepath('README',root_dir='.',subdir=dir)
openw,1,filename
printf,1,'The sequence of png files of this directory'
printf,1,'has been produced using the following IDL command:'
printf,1,comline
printf,1,"This command made use of the routine '"+plotting_proc+"'"
printf,1,'Its source has been copied to the present directory'
printf,1,''
printf,1,'We give below two examples that encode the png files sequence'
printf,1,'into video files that can be read straight away with Powerpoint'
printf,1,'Example 1:'
printf,1,'ffmpeg -f image2 -i movie%07d.png -qscale 1 out.mpg'
printf,1,'This produces an MPEG-1 video file called out.mpg'
printf,1,'Note that its quality is specified by -qscale'
printf,1,'1 is for excellent, 31 is for poor'
printf,1,'If this parameter is not specified, the bitrate remains'
printf,1,'constant and the quality drops quickly after the first frames'
printf,1,''
printf,1,'Example 2:'
printf,1,'ffmpeg -f image2 -i movie%07d.png -qscale 1 -vcodec mpeg4 out.mp4'
printf,1,'This produces an Apple QuickTime movie file (ftyp) that can'
printf,1,'be also read directly with Powerpoint on Mac OS X'
close,1
if (titleison) then begin
    tsize = (size(d_title))[1]
    for i=0, t_duration*24 do begin
        call_procedure, plotting_proc, 0.0
        field = tvrd()
	if NOT(keyword_set(noreduce)) then begin
           field = (field*254L)/255
        endif
        tv,field
        for j=0, tsize-1 do begin
            xyouts, 0.5, 0.6+0.05*tsize-.1*j, d_title[j],$
                   alignment = 0.5,/norm,color=255
        endfor
        fieldtitle = tvrd()
        blending = min([i/24.0/d_ramp,1.0])
        tv,field*(1.0-blending)+fieldtitle*blending
        if (signison) then begin
            if (i GT (t_duration-s_duration)*24) then begin
                xyouts, 0.95,0.1,sign,alignment=1.0,/norm
            endif
        endif
        field = tvrd()
        ext=strmid('0000000000'+strcompress(string(counter),/remove_all),6,7,/r)
        counter=counter+1
        filename=filepath('movie'+ext+'.png',root_dir='.',subdir=dir)
        write_png,filename,field,red,green,blue
    endfor
    signison = 0
endif
for i=0, d_nframe do begin
    call_procedure, plotting_proc, i*1.0/d_nframe
    field = tvrd()
    if NOT(keyword_set(noreduce)) then begin
       field = (field*254L)/255
    endif
    tv,field
    if (signison) then begin
        if (i LT (s_duration*24)) then begin
            xyouts, 0.95,0.1,sign,alignment=1.0,/norm
        endif
    endif
    field = tvrd()
    ext=strmid('0000000000'+strcompress(string(counter),/remove_all),6,7,/r)
    counter=counter+1
    filename=filepath('movie'+ext+'.png',root_dir='.',subdir=dir)
    write_png,filename,field,red,green,blue
endfor
print,'The sequence has been written in '+dir
set_plot,'x'
return
end
