pro read_desc, nb=nb, cpugrid=cpugrid, dir=dir, verbose=verbose,$
               volume=vol,$
               inter1=i1,$
               inter2=i2,$
               inter3=i3,$
               center1=c1,$
               center2=c2,$
               center3=c3,$
               x=x,y=y,z=z,radius=radius,$
               axial=axial,$
               colatitude=colatitude,$
               azimuth=azimuth,help=help,$
               edge1=e1, edge2=e2, edge3=e3,$
               lastlevel=lastlevel,$
               coordtype=coordtype, coordpermut=coordpermut, level=level, size=size
common HD_param, str_par
;+
; NAME:
;       READ_DESC
;
; PURPOSE: 
;       Read or reconstruct any field value in a cpugrid descriptor
;       (such a zone volumes, interface surfaces, center position,
;       presence of refined zones in a zone, etc.) and gives any
;       combination (one can ask for instance for the 'x' value of the
;       zones centers for a spherical mesh).
;
; CATEGORY:
;       Input routine
;
; CALLING SEQUENCE:
;       READ_DESC, CPUGRID=cpugrid [,NB=nb] [,DIR=dir] [,/VERBOSE]
;       [,VOLUME=vol] [,INTER1=i1] [,INTER2=i2] [,INTER3=i3]
;       [,CENTER1=c1] [,CENTER2=c2] [,CENTER3=c3] [,X=x] [,Y=y] [,Z=z]
;       [,RADIUS=rad] [,AXIAL=ax] [,COLATITUDE=col] [,AZIMUTH=az]
;       [,EDGE1=e1] [,EDGE2=e2] [,EDGE3=e3] [,LASTLEVEL=last]
;       [,COORDTYPE=type] [,COORDPERMUT=permut] [,LEVEL=level]
;       [,SIZE=size] [, FAST=fast] [,/HELP]
;
; 
; NB=nb: output number for which the cpugrid descriptor is to be
; read. Defaults to 0 or sticky value.
;
; CPUGRID=cpugrid: the cpugrid descriptor number 'cpugrid' to be
; read. This argument is mandatory.
;
; DIR=dir: looks for the cpugrid descriptor located at
; 'dir'. Default is '.'
;
; /VERBOSE: makes verbose standard output.
;
; VOLUME=vol: upon return 'vol' is a double precision 3D arrays that
; contains the zone volumes. Only the active part of the cpugrid
; descriptor is returned (the ghost zones are discarded).
;
; INTER1=i1: same as VOLUME but upon return the array contains the
; surface of the zone interfaces perpendicular to the first dimension
;
; INTER2=i2 or INTER3=i3: same as before but for 2nd and 3rd dimension
; respectively.
;
; CENTERi=ci: same as INTERi but contains the coordinate of the zone
; center instead of the surface of the zone interface. i refers to the
; dimension.
;
; X=x....AZIMUTH=az: upon return the corresponding variable contains a
; double precision 3D arrays that contains the desired coordinate of
; the zone center, regardless of the geometry (one can therefore ask
; for the 'X' field of the zones of a cylindrical mesh). The meaning
; of all these keywords is unambiguous. AXIAL_RADIUS is the distance
; to the 'z' axis. RADIUS is the distance to the origin. AXIAL_RADIUS
; is therefore the 'radius' coordinate in cylindrical geometry, while
; RADIUS is the 'radius' coordinate in spherical geometry.
;
; EDGE1=e1: upon return e1 is a double precision vector which contains
; the first coordinate of the zone interfaces perpendicular to the
; first dimension. If the cpugrid has size N1xN2xN3, e1 has size N1+1.
;
; EDGE2=e2 or EDGE3=e3: same as previously but for 2nd and 3rd
; coordinate respectively.
;
; LASTLEVEL=last: upon return the array 'last' is a 3D double
; precision floating point array in which each zone contains either
; 1.0 if it is not covered by a higher resolution mesh or 0.0 if it is
; covered by a higher resolution mesh.
;
; COORDTYPE=coordtype: upon return 'coordtype' is either 0 for a
; cartesian geometry, 1 for a cylindrical geometry or 2 for a
; spherical geometry.
;
; COORDPERMUT=coordpermut: upon return 'coordpermut' is a string that is
; a permutation of 123 which describes the ordering of
; coordinates. See code manual for details.
;
; LEVEL=level: upon return 'level' contains the level of
; the cpugrid.
;
; SIZE=size: upon return 'size' is an integer 3-vector that contains
; the size of the cpugrid (ghosts excluded)
;
; /HELP: print this help
;-
if ((n_elements(cpugrid) EQ 0) OR (keyword_set(help))) then begin
    doc_library, 'read_desc'
    return
endif
if (NOT(keyword_set(verbose))) then verbose=0
readstruct, nb=nb, dir=dir
p=str_par.sdir.in
g=str_par.sgrid
coordinates_name = [['X', 'Y', 'Z'],$
                       ['Radius', 'Azimuth', 'Z'],$
                       ['Radius', 'Azimuth', 'Colatitude']]
if verbose then begin
    print,p.coordtype+" geometry"
endif
if p.coordtype EQ "Cartesian" then coordtype=0
if p.coordtype EQ "Cylindrical" then coordtype=1
if p.coordtype EQ "Spherical" then coordtype=2
coord_perm = lonarr(3)
coordpermut = p.coordpermut
for i=0,2 do begin
    coord_perm(i) = long(strmid(coordpermut,i,1))-1
    if (verbose) then begin
        print, strcompress('Coordinate number'+string(i+1)+ ' is '+coordinates_name(coord_perm(i), coordtype))
    endif
endfor
index = 0
nbg = g.nb_cpu_grids
while ((index LT nbg) AND (g.cgridnumber(index) NE cpugrid)) do index=index+1
pindex = g.parent(index)
e1=reform(g.edges(pindex,0,*))
e2=reform(g.edges(pindex,1,*))
e3=reform(g.edges(pindex,2,*))
size=reform(g.cpugsize(index,*))
nx=size[0]
ny=size[1]
nz=size[2]
ngh = g.nghost
e1=e1(ngh+g.pcorner_min[index,0]:ngh+g.pcorner_max[index,0])
e2=e2(ngh+g.pcorner_min[index,1]:ngh+g.pcorner_max[index,1])
e3=e3(ngh+g.pcorner_min[index,2]:ngh+g.pcorner_max[index,2])
if verbose then print, strcompress("Size in cells:"+string(nx)+" *"+string(ny)+" *"+string(nz))
level=g.cgridlevel(index)
i1=e1
i2=e2
i3=e3
if verbose then print, strcompress("Level is:"+string(level))
if verbose then begin
    print, strcompress(coordinates_name(coord_perm(0), coordtype)+" goes from"+string(min(e1))+" to"+string(max(e1)))
    print, strcompress(coordinates_name(coord_perm(1), coordtype)+" goes from"+string(min(e2))+" to"+string(max(e2)))
    print, strcompress(coordinates_name(coord_perm(2), coordtype)+" goes from"+string(min(e3))+" to"+string(max(e3)))
endif
return
case Coordtype of
    0: begin
        x=reform(center(q(4),*,*,*))
        y=reform(center(q(5),*,*,*))
        z=reform(center(q(6),*,*,*))
        axial=sqrt(x*x+y*y)
        radius=sqrt(x*x+y*y+z*z)
        azimuth=atan(y,x)
        colatitude=atan(axial,z)
    end
    1: begin
        axial=reform(center(q(4),*,*,*))
        azimuth=reform(center(q(5),*,*,*))
        z=reform(center(q(6),*,*,*))
        x=axial*cos(azimuth)
        y=axial*sin(azimuth)
        radius=sqrt(axial*axial+z*z)
        colatitude=atan(axial,z)
    end
    2: begin
        radius=reform(center(q(4),*,*,*))
        azimuth=reform(center(q(5),*,*,*))
        colatitude=reform(center(q(6),*,*,*))
        axial=radius*sin(colatitude)
        z=radius*cos(colatitude)
        x=axial*cos(azimuth)
        y=axial*sin(azimuth)
    end
endcase
return
end
