pro read_field, nb=nb, radix=radix, component=component,$
                cpugrid=cpugrid, dir=dir, field=field,$
                center1=center1, center2=center2, center3=center3,$
                edge1=edge1, edge2=edge2, edge3=edge3,$
                quiet=quiet, bounds=bounds, level=level, size=size,$
                velocity=velocity,mesh=mesh,diff=diff, mirror=mirror
common HD_param, str_par
;+
; NAME:
;       READ_FIELD
;
; PURPOSE:
;       Read a hydrodynamics field corresponding to a specific output
;       number and a specific cpugrid number. The HD field can be
;       any field, i.e. density or a velocity component, or any other
;       additionnal field that can be specified by its suffix. It can
;       also return the edges value of the mesh, or the center values
;       (1D arrays taken as the arithmetic mean of successive edges),
;       the size of the array, its level (refinement level), and
;       the boundaries of the field.
;
; CATEGORY:
;       Input routine
;
; CALLING SEQUENCE:
;       READ_FIELD, CPUGRID=cpugrid [, RADIX=radix] [, COMPONENT=component]
;       [, NB=nb] [, DIR=dir] [, FIELD=field] [,
;       CENTER1=center1] [, CENTER2=center2] [, CENTER3=center3]
;       [, EDGE1=edge1] [, EDGE2=edge2] [, EDGE3=edge3] [,/QUIET] 
;       [,BOUNDS=bounds] [, LEVEL=level] [,SIZE=size]
;       [,/VELOCITY] [,DIFF=diff] [,/MIRROR]
; 
; CPUGRID=cpugrid: specifies the cpugrid number to be
; read. Mandatory argument.
;
; NB=nb: output #nb is read. Defaults to 0 or sticky value.
;
; RADIX: uses the default radix 'density'. Can be changed to
; 'velocity' to display a velocity component, or any other existing field.
;
; COMPONENT: when the 'velocity' radical is active, component defaults
; to 0. Can be changed to 1 or 2. 
;
; DIR: the dir relative to the current working directory where to
; look for the arrays. Defaults to '.' (current working directory).
;
; FIELD=field: upon return the field variable contains the HD field.
;
; CENTER(1,2,3)=centeri: upon return the centeri variable contains
; the center value for the ith coordinate. If the HD field has size
; N1xN2xN3, then the center array has size Ni.
;
; EDGE(1,2,3)=edgei: upon return the edgei variable contains the edge
; value for the ith coordinate. If the HD field has size N1xN2xN3,
; the the edge array has size Ni+1.
;
; /QUIET: does not write anything to the standard output.
;
; BOUNDS=bounds: upon return the 'bounds' variable will be a 6-vector
; that will contain:
; [edge1_min,edge1_max,edge2_min,edge2_max,edge3_min,edge3_max]
;
; LEVEL=level: upon return the 'level' variable will be
; an integer that is the corresponding level (i.e. refinement
; level) of the cpugrid.
;
; SIZE=size: upon return contains an integer 3-vector that is the size
; of the HD field.
;
; DIFF=diff: if this keyword is set and is larger than or equal to 0, 
; the same cpugrid field read for output number 'diff' is substracted
; from the field read for output number 'nb'. This is useful to see
; differences on slightly perturbed fields. This works if and only if
; the same mesh structure has been used for outputs number 'nb' and 'diff',
; and if they have been obtained with the same number of CPUs.
;
; /VELOCITY: equivalent to specify: RADIX='velocity'
;
; /MIRROR: At the present time this keyword only works if the coordinate
; system is SPHERICAL and if the Co-latitude corresponds to the third
; dimension. In this case, instead of returning the field directly, it
; takes its mirror symmetry wrt to the third (ie, colatitude) dimension, 
; and it affects the field EDGES3, which is set to !PI-z, where 'z' is
; the original list of zone edges in colatitude prior to the equatorial
; symmetry.
;-
if (n_elements(cpugrid) EQ 0) then begin
    doc_library, 'read_field'
    return
endif
if n_elements(diff) EQ 0 then diff=-1
if NOT(keyword_set (radix)) then radix='density'
if keyword_set (velocity) then radix='velocity'
if NOT(keyword_set (component)) then component=0
if keyword_set (quiet) then verbose = 0 else verbose = 1
read_desc, cpugrid=cpugrid, nb=nb, dir=dir, size=size,$
           edge1=x,edge2=y,edge3=z,level=lev
subdir = str_par.sgrid.subdir
swap = str_par.sdir.swap_endian
number = str_par.noutput
nx = size[0]
ny = size[1]
nz = size[2]
field=dblarr(nx,ny,nz)
if (keyword_set(mirror)) then begin
    if (str_par.sgrid.coordname[2] NE 'Co-latitude') then begin
        print, 'Error : mirror effect only implemented in spherical'
        print, 'coordinates, and if third dimension is colatitude'
    endif else begin
        field=reverse(field,3)
        z=!PI-z
    endelse
endif
bounds = [x[0],y[0],z[0],x[nx],y[ny],z[nz]]
edge1=x
edge2=y
edge3=z
center1=.5*(x[0:nx-1]+x[1:nx])
center2=.5*(y[0:ny-1]+y[1:ny])
center3=.5*(z[0:nz-1]+z[1:nz])
if verbose then print,$
  strcompress('Array size is '+string(nx)+' * '+string(ny)+' * '+string(nz))
if verbose then print, 'min/max zone center on coord. 1: ', x(0), x(nx)
if verbose then print, 'min/max zone center on coord. 2: ', y(0), y(ny)
if verbose then print, 'min/max zone center on coord. 3: ', z(0), z(nz)
filename = 'gas'+radix+$
           string(number)+'_'+string(cpugrid)+'_'+string(lev)+'.dat'
filename = strcompress (filename, /remove_all)
if verbose then print, "Reading "+filename
openr, 1, filepath (filename, ROOT_dir='.', SUBDIR=subdir)
cp = component
while (cp NE 0) do begin 
    cp = cp-1
    readu, 1, field
endwhile
readu, 1, field
if swap then field=swap_endian(field)
close, 1
if (keyword_set(mirror)) then begin
    if (str_par.sgrid.coordname[2] NE 'Co-latitude') then begin
        print, 'Error : mirror effect only implemented in spherical'
        print, 'coordinates, and if third dimension is colatitude'
    endif else begin
        field=reverse(field,3)
    endelse
endif
if (diff GE 0) then begin
    field0 = field
    nbch0 = strcompress(string(diff),/remove_all)
    outd = strmid("output00000",0,11-strlen(nbch0))+nbch0
    subdirdiff = str_par.sgrid.dir+'/'+outd
    filenamediff = 'gas'+radix+$
                   string(diff)+'_'+string(cpugrid)+'_'+string(lev)+'.dat'
    filenamediff = strcompress (filenamediff, /remove_all)
    openr, 1, filepath (filenamediff, ROOT_dir='.', SUBDIR=subdirdiff)
    cp = component
    while (cp NE 0) do begin 
        cp = cp-1
        readu, 1, field0
    endwhile
    readu, 1, field0
    if swap then field0=swap_endian(field0)
    close, 1
    field = field-field0
endif
if ((size(field))(0) EQ 1) then field=reform(field,(size(field))(1), 1, 1)
if ((size(field))(0) EQ 2) then field=reform(field,(size(field))(1),(size(field))(2), 1)
mesh={field:total(field,3),xgrid:x[0:nx],ygrid:y[0:ny]}
return
end
