pro read_vel, nb=nb, descriptor=descriptor, path=path,$
                center1=x, center2=y, center3=z,$
                edge1=edgex, edge2=edgey, edge3=edgez,$
                quiet=quiet, bounds=bounds,$
                generation=generation, size=sz,$
                vx=vx, vy=vy, vz=vz, vrad=vrad, vaxial=vaxial,$
                vcolatitude=vcol, vazimuthal=vazim,field=field
;+
; NAME:
;       READ_VEL
;
; PURPOSE:
;
;       Read a velocity field corresponding to a specific output
;       number and a specific descriptor number.  It can also return
;       the edges value of the mesh, or the center values (1D arrays
;       taken as the arithmetic mean of successive edges), the size of
;       the array, its generation (refinement level), and the
;       boundaries of the field. The velocity field is referred to by
;       any of its component in an arbitrary geometry. For instance,
;       it is possible to read the VZ component in a spherical
;       coordinate system.
;
; CATEGORY:
;       Input routine
;
; CALLING SEQUENCE:

;       READ_VEL, NB=nb [, DESCRIPTOR=descriptor] [, PATH=path] [,
;       CENTER1=center1] [, CENTER2=center2] [, CENTER3=center3] [,
;       EDGE1=edge1] [, EDGE2=edge2] [, EDGE3=edge3] [,/QUIET]
;       [,BOUNDS=bounds] [, GENERATION=generation] [,SIZE=size]
;       [,VX=vx] [,VY=vy] [,VZ=vz] [,VRAD=vrad] [,VAXIAL=vaxial]
;       [,VCOLATITUDE=vcol] [,VAZIMUTHAL=vazim] [,FIELD=field]
; 
; NB=nb: output #nb is read.
;
; DESCRIPTOR=descriptor: specifies the descriptor number to be
; read. Defaults to 0 (Base mesh).
;
; PATH: the path relative to the current working directory where to
; look for the arrays. Defaults to '.' (current working directory).
;
; CENTER(1,2,3)=centeri: upon return the centeri variable contains
; the center value for the ith coordinate. If the HD field has size
; N1xN2xN3, then the center array has size Ni.
;
; EDGE(1,2,3)=edgei: upon return the edgei variable contains the edge
; value for the ith coordinate. If the HD field has size N1xN2xN3,
; the the edge array has size Ni+1.
;
; /QUIET: does not write anything to the standard output.
;
; BOUNDS=bounds: upon return the 'bounds' variable will be a 6-vector
; that will contain:
; [edge1_min,edge1_max,edge2_min,edge2_max,edge3_min,edge3_max]
;
; GENERATION=generation: upon return the 'generation' variable will be
; an integer that is the corresponding generation (i.e. refinement
; level) of the descriptor.
;
; SIZE=size: upon return contains an integer 3-vector that is the size
; of the HD field.
;
; VX=vx to VAZIMUTHAL=vazimuthal: upon return the corresponding
; variable contains a double precision 3D array that represents a
; component of the velocity field. The keyword name is unambiguous.
; See also the help of READ_DESC.
;
; FIELD=field: upon return, contains a N1xN2xN3x3 double precision
; array that contains the raw velocity file as written by the code.
;-

   if (n_elements(nb) EQ 0) then begin
       doc_library, 'read_vel'
       return
   endif
number=nb
if keyword_set (descriptor) then descriptor=descriptor else descriptor=0
if keyword_set (path) then path=path else path='.'
if keyword_set (quiet) then verbose = 0 else verbose = 1
coordfile = filepath ('.coord.dat', subdir=path, root_dir='.')
openr, 1, coordfile, ERROR = err
readf, 1, coordtype
close, 1
spawn,'grep -i coordpermut '+$
  findfile(filepath('params.dat', ROOT_dir='.', SUBDIR=path)),$
  /sh,res
perm=strmid(res,strlen(res)-3)
coordpermut=perm
coord_perm=lonarr(3)
coordinates_name = [['X', 'Y', 'Z'],$
                       ['Radius', 'Azimuth', 'Z'],$
                       ['Radius', 'Azimuth', 'Colatitude']]
if verbose then begin
   case coordtype of
       0: print, 'Cartesian geometry'
       1: print, 'Cylindrical geometry'
       2: print, 'Spherical geometry'
   endcase
endif
for i=0,2 do begin
    coord_perm(i) = long(strmid(perm,i,1))-1
    if (verbose) then begin
        print, 'Coordinate number ', i+1, ' is '+coordinates_name(coord_perm(i), coordtype)
    endif
endfor
descfile = strcompress('fulldesc'+string(descriptor)+'_*.dat', /remove_all)
strres = findfile (filepath(descfile, ROOT_dir='.', SUBDIR=path))
if ((size(strres))(0) EQ 0) then begin
    r = dialog_message(descfile+' does not exist in '+path, /error, title='Warning')
    return
endif
nb = (size(strres))(1)
openr, 1, strres(0)
q=lonarr(7)
readu,1,q
generation = q(0)
sz=lonarr(3)
sz(0)=q(1)
sz(1)=q(2)
sz(2)=q(3)
nx = q(1)
ny = q(2)
nz = q(3)
field=dblarr(nx,ny,nz,3)
edges_x=dblarr(nx+5)
edges_y=dblarr(ny+5)
edges_z=dblarr(nz+5)
readu,1,edges_x,edges_y,edges_z
x=.5*(edges_x(2:nx+1)+edges_x(3:nx+2))
y=.5*(edges_y(2:ny+1)+edges_y(3:ny+2))
z=.5*(edges_z(2:nz+1)+edges_z(3:nz+2))
edgex = edges_x(2:nx+2)
edgey = edges_y(2:ny+2)
edgez = edges_z(2:nz+2)
bounds = dblarr(6)
bounds(0) = edges_x(2)
bounds(1) = edges_x(nx+2)
bounds(2) = edges_y(2)
bounds(3) = edges_y(ny+2)
bounds(4) = edges_z(2)
bounds(5) = edges_z(nz+2)
foo = dblarr(nx+4,ny+4,nz+4)
center = dblarr(nx+4,ny+4,nz+4,3)
readu,1,foo
readu,1,foo
readu,1,foo
readu,1,foo
center(*,*,*,0)=foo
readu,1,foo
readu,1,foo
center(*,*,*,1)=foo
readu,1,foo
readu,1,foo
center(*,*,*,2)=foo
close,1
center=center(2:nx+1,2:ny+1,2:nz+1,*)
if (verbose EQ 1) then print, strcompress('Array size is '+string(nx)+' * '+string(ny)+' * '+string(nz))
if (verbose EQ 1) then print, 'min/max zone center on coord. 1: ', x(0), x(nx-1)
if (verbose EQ 1) then print, 'min/max zone center on coord. 2: ', y(0), y(ny-1)
if (verbose EQ 1) then print, 'min/max zone center on coord. 3: ', z(0), z(nz-1)
filename = 'gasvelocity'+string(number)+'_'+string(descriptor)+'_'+string(q(0))+'.dat'
filename = strcompress (filename, /remove_all)
openr, 1, filepath (filename, ROOT_dir='.', SUBDIR=path)
readu, 1, field
close, 1
case Coordtype of
    0: begin
        vx=reform(field(*,*,*,q[4]))
        vy=reform(field(*,*,*,q[5]))
        vz=reform(field(*,*,*,q[6]))
        x=reform(center(*,*,*,q[4]))
        y=reform(center(*,*,*,q[5]))
        z=reform(center(*,*,*,q[6]))
        if Arg_Present(vrad) then vrad=(vx*x+vy*y+vz*z)/sqrt(x*x+y*y+z*z)
        if Arg_Present(vaxial) then vaxial=(vx*x+vy*y)/sqrt(x*x+y*y)
        if Arg_Present(vcol) then print,'vcol for cartesian : Not implemented yet !!!'
        if Arg_Present(vazim) then print,'vazim for cartesian : Not implemented yet !!!'
    end
    1: begin
        vaxial=reform(field(*,*,*,q[4]))
        vazim=reform(field(*,*,*,q[5]))
        vz=reform(field(*,*,*,q[6]))
        axial_radius=reform(center(*,*,*,q[4]))
        azimuth=reform(center(*,*,*,q[5]))
        z=reform(center(*,*,*,q[6]))
        if Arg_Present (vrad) then print, 'vrad for cylindrical: Not implemented yet !!!'
        if Arg_Present (vx) then print, 'vx for cylindrical: Not implemented yet !!!'
        if Arg_Present (vy) then print, 'vy for cylindrical: Not implemented yet !!!'
        if Arg_Present (vcol) then print, 'vcol for cylindrical: Not implemented yet !!!'
    end
    2: begin
        vrad=reform(field(*,*,*,q[4]))
        vazim=reform(field(*,*,*,q[5]))
        vcol=reform(field(*,*,*,q[6]))
        rad=reform(center(*,*,*,q[4]))
        azim=reform(center(*,*,*,q[5]))
        colat=reform(center(*,*,*,q[6]))
        if Arg_Present (vx) then vx=vrad*sin(colat)*cos(azim)+rad*vcol*cos(colat)*cos(azim)- $
                                    rad*vazim*sin(colat)*sin(azim)
        if Arg_Present (vy) then vy=vrad*sin(colat)*sin(azim)+rad*vcol*cos(colat)*sin(azim)+ $
                                    rad*vazim*sin(colat)*cos(azim)
        if Arg_Present (vz) then vz=vrad*cos(colat)-rad*vcol*sin(colat)
        if Arg_Present (vaxial) then print,'vaxial for spherical: Not implement yet !!!'
    end
endcase
return
end


