pro SHELL_vel, radius=radius, nb=nb, genmin=genmin,$
 genmax=genmax, symmetry=symmetry, path=path, verbose=verbose, _extra=e
;+
; NAME:
;       SHELL_VEL
;
; PURPOSE:
;
;       Cut a 2D radial slice (shell-like) out of a spherical mesh or
;       a set of nested spherical meshes, and displays the
;       corresponding velocity field (v_azimuth,v_colatitude).
;
; CATEGORY:
;       Plotting routine
;
; CALLING SEQUENCE:
;
;       SHELL_VEL, RADIUS=r, NB=nb [,GENMIN=gmin]
;       [,GENMAX=gmax] [,PATH=path] [,/VERBOSE]
;
; NOTE: in addition to these keywords, EQUATORIAL_VEL inherits all the
; keywords specific to the PLOT2D_VEL routine (see its helpfile).
; 
; RADIUS=r: the radius at which the shell-like slice must be cut.
;
; NB=nb: the output number of the data that must be used.
;
; GENMIN=gmin: only meshes with generation larger than 'gmin' are
; taken into account.
;
; GENMAX=gmax: only meshes with generation smaller than 'gmax' are
; taken into account.
;
; /SYMMETRY: if this keyword is set, the mesh is doubled up in
; colatitude, assuming v_azimuth(-z) = v_azimuth(z) and vz(-z)=-vz(z)
;
; /VERBOSE: verbose output. Tells which descriptors are intercepted,
; where, and the min and max value of the velocity module.
;
; Note: as in SHELLCUT, the vertical axis label is !PI/2-colatitude
; instead of colatitude. SHELL_VEL can be used with the /NOERASE
; keyword after a call to SHELLCUT.
;-
if (n_elements(nb) EQ 0) OR (n_elements(radius) EQ 0) then begin
    doc_library, 'shell_vel'
    return
endif
if keyword_set (path) then path=path else path='.'
if (n_elements(genmin) EQ 0) then genmin = 0
if (n_elements(component) EQ 0) then component = 0
if (n_elements(genmax) EQ 0) then genmax = genmin+8
desclist, generation=indgen(genmax-genmin+1)+genmin, nb=nb, list=list, path=path
list=[list]
vx_list=[0.0]
vy_list=[0.0]
xx=[0.0]
yy=[0.0]
if (list(0) NE -1) then begin
    for k=0,(size(list))(1)-1 do begin
        read_vel, nb=nb, desc=list(k),path=path,VAZIMUTHAL=vazim,VCOL=vcol, /quiet
        read_desc,desc=list(k),path=path,azimuth=az, col=col, lastlevel=mask, gen=gen, edge2=rad
        vx = vazim
        vy = -vcol   ; we say -vcol because we plot azim vs. !PI/2 MINUS colatitude.
        x_slice = reform(az(*,0,*))
        z_slice = reform(col(*,0,*))
        cut_field,vx,edgeperp=rad,cutdim=1,cutvalue=radius,overlap=overlap,slice_vx
        cut_field,vy,edgeperp=rad,cutdim=1,cutvalue=radius,overlap=overlap,slice_vy
        if (overlap EQ 1) then begin
            if keyword_set(verbose) then print, 'Descriptor ', list(k),' intercepted'
            cut_field,mask,edgeperp=rad,cutdim=1,cutvalue=radius,maskslice
            if (gen LT genmax) then begin
                b = where(maskslice GT 0.5)
                vx_list=[vx_list,slice_vx(b)]
                vy_list=[vy_list,slice_vy(b)]
                xx=[xx,x_slice(b)]
                yy=[yy,z_slice(b)]
            endif else begin
                b = where(maskslice*0.0 LT 0.1)
                vx_list=[vx_list,slice_vx(b)]
                vy_list=[vy_list,slice_vy(b)]
                xx=[xx,x_slice(b)]
                yy=[yy,z_slice(b)]
            endelse
        endif
    endfor
endif
if ((size(xx))(1) EQ 1) then begin
    print, 'No mesh intercepted'
    return
endif
xx=xx(1:*)
yy=yy(1:*)
vx=vx_list(1:*)
vy=vy_list(1:*)
yy=!PI/2.0-yy
if keyword_set(verbose) then print, 'Maximum velocity value = ', max(sqrt(vx*vx+vy*vy))
if keyword_set(verbose) then print, 'Minimum velocity value = ', min(sqrt(vx*vx+vy*vy))
if keyword_set(symmetry) then begin
    xx=[xx,xx]
    yy=[yy,-yy]
    vx=[vx,vx]
    vy=[vy,-vy]
endif
!z.type=0
plot2d_vel,vx,vy,xx,yy,_extra=e
return
end
