pro slice, dir=dir, nb=nb,levels=levels,coordperp=coordperp,dim=dim,$
           radix=radix,component=component,showcpus=showcpus,$
           xrange=xrange,yrange=yrange,showgrids=showgrids,auto=auto,$
           func=func,velocity=velocity,min=min,max=max,grid=grid,$
           box=box,cpugrids=cpugrids,light=light,help=help,nofield=nofield,$
           diff=diff,mirror=mirror,nogridnumber=nogridnumber,_extra=e
common HD_param, str_par
;+
; NAME:
;       SLICE
;
; PURPOSE:
;
;       Plot a  2D slice of a scalar quantity on a set of
;       cpugrids, for a given output
;
; CATEGORY:
;       Plotting routine
;
; CALLING SEQUENCE:
;
;       SLICE [,DIR=dir] [,NB=nb] [,LEVELS=levels] [,COORDPERP=coordperp]
;       [,DIM=dim] [,/SHOWCPUS] [,/SHOWGRIDS] [,/AUTO] [,/GRID] [,/BOX]
;       [,/LIGHT] [,/HELP] [,FUNC=func] [,XRANGE=xrange] [,YRANGE=yrange]
;       [,MIN=min] [,MAX=max] [,CPUGRIDS=cpugrids] [,RADIX=radix]
;       [,COMPONENT=component] [,/NOFIELD] [,/VELOCITY] [,DIFF=diff]
;       [,/MIRROR] [,/NOGRIDNUMBER]
;
; DIR=dir: the directory where to look for the files. Defaults to '.',
; and is sticky.
;
; NB=nb: the output number of the data that must be used. Defaults to
; 0, and is sticky.
;
; LEVELS=levels: vector list of levels to be plotted. Defaults to all
; levels. Note that LEVELS=[0,3] means level 0 and level 3, not level
; 0 to 3. For this you should enter LEVELS=indgen(4).
;
; COORDPERP=coordperp: the value of the coordinate perpendicular to
; the cut at which one wants to have a slice. Defauts to 0, i.e. if DIM
; is set to its default (see below) the slice is a (X,Y) slice of the
; system at Z=0, unless the coordinate system is spherical and DIM is
; set to 2 (the default), in which case it defaults to !PI/2.-1d-5,
; corresponding to an equatorial cut.
;
; DIM=dim: 'dim' has to be an integer between 0 and 2, and stands 
; for the dimension perpendicular to the slice. It defaults to 2,
; therefore the slice is in the (X,Y) plane for a cartesian grid
; with no coordinates reordering.
;
; /SHOWCPUS: if switched on, display all the edges of the CPUgrids
; that the slice intersects, as well as the CPU number and CPU grid
; number. If the resulting output is overloaded, one may consider
; using the /LIGHT switch, in which case only the CPU number is
; plotted.
;
; /SHOWGRIDS: if switched on, display all the edges of the Grids that
; the slice intersects, as well as the grid number near the center of
; each grid. The resulting plot is therefore independant of the number
; of CPUs used for the run. Note that /SHOWGRIDS and /SHOWCPUS can be
; used at the same time, although the resulting plot might be a bit
; hairy. 
;
; /AUTO: if switched on, the limits of the plot are automatically set
; so that it fits exactly within the axes limits. This implies that
; any previous content of the window is erased. This is the only case
; for which the window is previously erased. If /AUTO is not set, the
; behaviour is to always plot over the previous content.
;
; /GRID: if switched on, the grid cell limits are plotted over the
; grid content.
;
; /BOX: if switched on, a box with axes limits and names of the
; variables is added to the plot. 
;
; /LIGHT: useful only if /SHOWCPUS is present, in order to reduce the
; size of printed information for each CPUGrid.
;
; /HELP: prints this help.
;
; FUNC=func: if this keyword is set, 'func' has to be a string that
; represents the name of a function, that is applied to the data field
; prior to display, e.g. FUNC='alog'.
;
; XRANGE=xrange and YRANGE=yrange: set the X or Y-axis range to the
; value given by the 2-vector 'xrange' or 'yrange'. Implies /xstyle
; and /ystyle (i.e. the range is exactly the one specified by the user).
;
; MIN=min: sets the data field value corresponding to the lowest color
; level. Any data lower than this value is represented with the
; minimum color index. It defaults to the minimum value among all the
; slices to be displayed. When used in conjunction with 'FUNC', there
; is no need to change the value of MIN, func is automatically applied
; to the value entered by the user.
;
; MAX=max: sets the data field value corresponding to the highest color
; level. Any data larger than this value is represented with the
; maximum color index. It defaults to the maximum value among all the
; slices to be displayed. When used in conjunction with 'FUNC', there
; is no need to change the value of MAX, func is automatically applied
; to the value entered by the user.
;
; CPUGRIDS=cpugrids: if this keyword is present, 'cpugrids' has to be
; a vector set of the cpugrid numbers to be displayed. A CPUGrid,
; even intercepted by the slice, will be displayed only if its number
; belongs to 'cpugrids'. When used together with the LEVELS keyword,
; it implies that a cpugrid intercepted by the slice is displayed if
; and only if the number of this cpugrid belongs to the 'cpugrids'
; list AND the level of that cpugrid belongs to the 'levels' list.
;
; RADIX: uses the default radix 'density'. Can be changed to
; 'velocity' to display a velocity component, or any other existing field.
;
; COMPONENT: when the 'velocity' radical is active, component defaults
; to 0. Can be changed to 1 or 2. 
;
; /NOFIELD: does not plot any hydro data field, but plot everything
; else (axes, if any, mesh boundaries or zones, if specified, cpugrid
; boundaries, etc.). Useful to superimpose mesh boundary information
; to an existing plot.
;
; /VELOCITY: shortcut for "RADIX='velocity'"
;
; DIFF=diff: if this keyword is set and is larger than or equal to 0, 
; the slice with same characteristics for output number 'diff' is substracted
; from the slice performed for output number 'nb'. This is useful to see
; differences on slightly perturbed fields. This works if and only if
; the same mesh structure has been used for outputs number 'nb' and 'diff',
; and if they have been obtained with the same number of CPUs.
;
; /MIRROR: At the present time this keyword only works if the coordinate
; system is SPHERICAL and if the Co-latitude corresponds to the third
; dimension. In this case, the slice contains not only what it would
; contain without this keyword, but also the mirror symmetry  w.r.t. 
; the equator of the original slice.
;-
if keyword_set(help) then begin
    doc_library, 'slice'
    return
endif
readstruct,dir=dir,nb=nb
if n_elements(diff) EQ 0 then diff=-1
if n_elements(component) EQ 0 then component = 0
if n_elements(radix) EQ 0 then radix = "density"
if keyword_set(velocity) then radix="velocity"
mirmax = 0
if keyword_set (mirror) then mirmax = 1
if n_elements(levels) EQ 0 then levels=indgen(long(str_par.sgrid.maxlevel)+1)
levels=[levels]
if n_elements(cpugrids) EQ 0 then cpugrids=indgen(long(str_par.sgrid.nb_cpu_grids)+1)
if n_elements(dim) EQ 0 then dim=2
if n_elements(coordperp) EQ 0 then begin
    coordperp = 0.0
    p=str_par.sdir.in
    if p.coordtype EQ "Spherical" AND dim EQ 2 then begin
        coordperp = !PI/2.-1d-5
    endif
endif
dim1=(dim EQ 0)
dim2=2-(dim EQ 2)
;
;
;first scan
;
;
minval=1d30
maxval=-1d30
xmin=1d30
xmax=-1d30
ymin=1d30
ymax=-1d30
g=str_par.sgrid
for j=min(levels),max(levels) do begin
    if (([where(levels-j EQ 0)])(0) NE -1) then begin
        lev=[j]
        desclist,level=lev,nb=nb,list=list,dir=dir
        if (list[0] NE -1) then begin
            ncpugrids = (size(list))(1)
            for m=0,mirmax do begin
                for i=0, ncpugrids-1 do begin
                    if (([where(cpugrids-list(i) EQ 0)])(0) NE -1) then begin
                        read_field,field=f,edge1=x,edge2=y,edge3=z,size=s,$
                                   cpugrid=list(i),/quiet,radix=radix,$
                                   component=component,diff=diff,mirror=m
                        edges=dblarr(3,max(s)+1)
                        edges(0,0:s[0])=x
                        edges(1,0:s[1])=y
                        edges(2,0:s[2])=z
                        cut_field,f,sl,edgeperp=reform(edges(dim,0:s[dim])),$
                                  cutdim=dim,cutvalue=coordperp,overlap=ov,$
                                  /nointerp
                        if ov EQ 1 then begin
                            minx = min(edges(dim1,0:s[dim1]))
                            maxx = max(edges(dim1,0:s[dim1]))
                            miny = min(edges(dim2,0:s[dim2]))
                            maxy = max(edges(dim2,0:s[dim2]))
                            minv = min(sl)
                            maxv = max(sl)
                            if minx LT xmin then xmin=minx
                            if miny LT ymin then ymin=miny
                            if maxx GT xmax then xmax=maxx
                            if maxy GT ymax then ymax=maxy
                            if minv LT minval then minval=minv
                            if maxv GT maxval then maxval=maxv
                        endif
                    endif
                endfor
            endfor
        endif
    endif
endfor
if keyword_set(auto) then begin
    !x.range = [xmin,xmax]
    !y.range = [ymin,ymax]
    plot,[0,0],[1,1],/nodata,xstyle=5,ystyle=5,_extra=e
endif
if keyword_set(xrange) then begin
    !x.range = xrange
    plot,[0,0],[1,1],/nodata,xstyle=5,ystyle=5,/noerase,_extra=e
endif
if keyword_set(yrange) then begin
    !y.range = yrange
    plot,[0,0],[1,1],/nodata,xstyle=5,ystyle=5,/noerase,_extra=e
endif
PRINT, FORMAT = '($, "Min value: ")'
print, minval, Format='(E25.15)'
PRINT, FORMAT = '($, "Max value: ")'
print, maxval, Format='(E25.15)'
if n_elements(min) NE 0 then minval = min
if n_elements(max) NE 0 then maxval = max
;
;
;second scan
;
;
for j=min(levels),max(levels) do begin
    if (([where(levels-j EQ 0)])(0) NE -1) then begin
        lev=[j]
        desclist,level=lev,nb=nb,list=list,dir=dir
        if (list[0] NE -1) then begin
            ncpugrids = (size(list))(1)
            for m=0,mirmax do begin
                for i=0, ncpugrids-1 do begin
                    if (([where(cpugrids-list(i) EQ 0)])(0) NE -1) then begin
                        read_field,field=f,edge1=x,edge2=y,edge3=z,size=s,$
                                   cpugrid=list(i),/quiet,radix=radix,$
                                   component=component,diff=diff,mirror=m
                        p=g.parent(list(i))
                        ngh=g.nghost
                        edges=dblarr(3,max(s)+1)
                        edges(0,0:s[0])=x
                        edges(1,0:s[1])=y
                        edges(2,0:s[2])=z
                        cut_field,f,sl,edgeperp=reform(edges(dim,0:s[dim])),$
                                  cutdim=dim,cutvalue=coordperp,overlap=ov
                        if ov EQ 1 then begin
                            mesh={field:sl,$
                                  xgrid:reform(edges(dim1,0:s[dim1])),$
                                  ygrid:reform(edges(dim2,0:s[dim2]))}
                            if NOT(keyword_set(nofield)) then begin
                                displaymesh,mesh,min=minval,$
                                            max=maxval,grid=grid,func=func
                            endif
                            if keyword_set(showcpus) then begin
                                plots,min(mesh.xgrid),min(mesh.ygrid),/data
                                plots,max(mesh.xgrid),min(mesh.ygrid),$
                                      /data,/continue,_extra=e
                                plots,max(mesh.xgrid),max(mesh.ygrid),$
                                      /data,/continue,_extra=e
                                plots,min(mesh.xgrid),max(mesh.ygrid),$
                                      /data,/continue,_extra=e
                                plots,min(mesh.xgrid),min(mesh.ygrid),$
                                      /data,/continue,_extra=e
                                if NOT(keyword_set(light)) then begin
                                    xyouts,$
                                      .5*(min(mesh.xgrid)+max(mesh.xgrid)),$
                                      .5*(min(mesh.ygrid)+max(mesh.ygrid)),$
                                      /data,alignment=.5,$
                                      strcompress ("CPU#"+$
                                                   string(g.cpurank[list[i]])+$
                                                   "/CG#"+string(list[i]))
                                endif else begin
                                    xyouts,$
                                      .5*(min(mesh.xgrid)+max(mesh.xgrid)),$
                                      .5*(min(mesh.ygrid)+max(mesh.ygrid)),$
                                      /data,alignment=.5,$
                                      strcompress ("CPU"+$
                                                   string(g.cpurank[list[i]]))
                                endelse
                            endif
                            if keyword_set(showgrids) then begin
                                sizeg = [g.nx[p],g.ny[p],g.nz[p]]
                                plots,$
                                  g.edges[p,dim1,ngh],g.edges[p,dim2,ngh],/data
                                plots,$
                                  g.edges[p,dim1,sizeg[dim1]+ngh],$
                                  g.edges[p,dim2,ngh],/data,/continue,_extra=e
                                plots,$
                                  g.edges[p,dim1,sizeg[dim1]+ngh],$
                                  g.edges[p,dim2,sizeg[dim2]+ngh],$
                                  /data,/continue,_extra=e
                                plots,$
                                  g.edges[p,dim1,ngh],$
                                  g.edges[p,dim2,sizeg[dim2]+ngh],$
                                  /data,/continue,_extra=e
                                plots,g.edges[p,dim1,ngh],$
                                      g.edges[p,dim2,ngh],$
                                      /data,/continue,_extra=e
                                if (NOT(keyword_set(nogridnumber))) then begin
                                    xyouts,g.edges[p,dim1,ngh+.5*sizeg[dim1]],$
                                           g.edges[p,dim2,ngh+.5*sizeg[dim2]],$
                                           strcompress ("Grid #"+string(p))
                                endif
                            endif
                        endif
                    endif
                endfor
            endfor
        endif
    endif
endfor
if keyword_set(box) then begin
    plot,[0,0],[1,1],/nodata,xstyle=1,ystyle=1,/noerase,$
    xtitle=g.coordname[dim1],ytitle=g.coordname[dim2],_extra=e
endif
return
end
