pro torque, nb=nb, genmax=genmax, quiet=quiet,$
            verbose=verbose, total=total, inner=inner,$
            outer=outer, smoothing=smoothing, path=path,$
            symmetric=symmetric
;+
; NAME:
;       TORQUE
;
; PURPOSE:
;   Evaluate the specific torque exerted by the disk onto the planet, for a given
;   output number.
;
; CALLING SEQUENCE:
;       TORQUE, NB=number [, GENMAX=genmax] [,/QUIET] [,/VERBOSE]
;       [,TOTAL=total_torque] [,INNER=inner_torque]
;       [,OUTER=outer_torque] [,SMOOTHING=smoothing] [,PATH=path]
;       [,/SYMMETRIC]
; 
; NB=number: uses the output #number to evaluate the torque. Note that
; 'number' can be a vector, in which case the inner, outer and total
; torques are also vectors of the same length. It is about 10 to 20
; times faster to use this feature than to make a loop involving a
; call to the TORQUE procedure, since this latter has to re-read the
; large descriptor arrays each time.
;
; GENMAX=genmax: the meshes up to refinement level `genmax' are used
; to compute the torque. Example: TORQUE, NB=10, genmax=0 uses only
; the base mesh to evaluate the torque at output #10. Default is 8.
;
; QUIET: nothing is written on standard output. The default is to
; write the three torques values on the standard output (total disk
; torque, inner disk torque and outer disk torque, the first being the
; sum of the two last ones).
;
; VERBOSE: verbose message written on standard output.
;
; TOTAL=total_torque: the total torque value is written in the `total_torque' variable
;
; INNER=total_torque: the inner torque value is written in the `inner_torque' variable
;
; OUTER=total_torque: the outer torque value is written in the `outer_torque' variable
;
; SMOOTHING=smoothing: sets the length of the smoothing parameter used
; to evaluate the force. It should be the same as the smoothing length
; used at runtime. It defaults to 0.03.
;
; PATH=path: specifies the relative or absolute path where to find the
; output files. Defaults to '.'
;
; SYMMETRIC: set this keyword to multiply the result torques by
; two. Useful if only half of the disk vertical extend is described by
; the simulation, and reflexive boundary conditions are used at the equator.
;-
if (n_elements(nb) EQ 0) then begin
    doc_library, 'torque'
    return
endif
if (NOT(keyword_set(path))) then path='.'
if (n_elements(genmax) EQ 0) then genmax=8
if (n_elements(smoothing) EQ 0) then smoothing=0.03
if (NOT(keyword_set(verbose))) then verbose=0
if (NOT(keyword_set(quiet))) then quiet=0
if (genmax LT 0) then begin
    print, 'Error: incorrect value:'
    help, genmax
    return
end
spawn,'grep -i coordpermut '+$
  findfile(filepath('params.dat', ROOT_dir='.', SUBDIR=path)),$
  /sh,res
perm=strmid(res,strlen(res)-3)
coord_perm=lonarr(3)
for i=0,2 do begin
    coord_perm(i) = long(strmid(perm,i,1))-1
    if (verbose) then begin
        Case coord_perm(i) of
            0: print, 'Coordinate number ', i+1, ' is radius'
            1: print, 'Coordinate number ', i+1, ' is azimuth'
            2: print, 'Coordinate number ', i+1, ' is colatitude'
        endcase
    endif
endfor
nb=[nb]
len=(size(nb))(1)
inner=dblarr(len)
outer=dblarr(len)
total=dblarr(len)
for gen=0, genmax do begin
    descfile = strcompress('fulldesc*_'+string(gen)+'.dat', /remove_all)
    strres = findfile (filepath(descfile, ROOT_dir='.', SUBDIR=path))
    if ((size(strres))(0) EQ 0) then begin
        if (verbose) then print, 'No descriptor found for generation ', gen
    endif
    if ((size(strres))(0) NE 0) then begin
        numb = (size(strres))(1)
        for ii=0, numb-1 do begin
            descname = strres(ii)
            openr, 1, descname
            q=lonarr(7)
            readu,1,q
            generation=q(0)
            nx = q(1)
            ny = q(2)
            nz = q(3)
            field=dblarr(nx,ny,nz)
            edges_x=dblarr(nx+5)
            edges_y=dblarr(ny+5)
            edges_z=dblarr(nz+5)
            readu,1,edges_x,edges_y,edges_z
            volume=dblarr(nx+4,ny+4,nz+4)
            temp = volume
            radius = volume
            azimuth = volume
            colatitude = volume
            refined = volume
            readu,1,volume
            readu,1,temp
            for j=0,2 do begin
                readu,1,temp
                Case coord_perm(j) of
                    0: readu,1,radius
                    1: readu,1,azimuth
                    2: readu,1,colatitude
                endcase
            endfor
            readu,1,refined
            close,1
            volume=volume(2:nx+1,2:ny+1,2:nz+1)
            radius=radius(2:nx+1,2:ny+1,2:nz+1)
            azimuth=azimuth(2:nx+1,2:ny+1,2:nz+1)
            colatitude=colatitude(2:nx+1,2:ny+1,2:nz+1)
            refined=refined(2:nx+1,2:ny+1,2:nz+1)
            density = volume
            if (gen LT genmax) then begin
                inner_cells = where((radius LT 1.0) AND (refined LT -0.9))
                outer_cells = where((radius GE 1.0) AND (refined LT -0.9))
                inner_mask = 0.0*volume
                inner_mask(inner_cells) = 1.0
                outer_mask = 0.0*volume
                outer_mask(outer_cells) = 1.0
            endif else begin
                inner_cells = where(radius LT 1.0)
                outer_cells = where(radius GE 1.0)
                inner_mask = 0.0*volume
                inner_mask(inner_cells) = 1.0
                outer_mask = 0.0*volume
                outer_mask(outer_cells) = 1.0
            endelse
            mask_in=(volume*radius*sin(azimuth)*sin(colatitude)/$
                     ((radius^2+1.0-2.0*radius*cos(azimuth)*sin(colatitude)+$
                       smoothing*smoothing)^1.5)*inner_mask)
            mask_out=(volume*radius*sin(azimuth)*sin(colatitude)/$
                     ((radius^2+1.0-2.0*radius*cos(azimuth)*sin(colatitude)+$
                       smoothing*smoothing)^1.5)*outer_mask)
            for k=0, len-1 do begin
                densname = strcompress('gasdensity'+string(nb(k))+'_'+$
                                       strmid(descname,strpos(descname,$
                                                              'fulldesc')+8),$
                                       /remove_all)
                densname = filepath(densname, ROOT_dir='.', SUBDIR=path)
                if Verbose then print, 'Reading ',densname
                openr,1,densname
                readu,1,density
                close,1
                it_incr = total(mask_in*density)
                ot_incr = total(mask_out*density)
                inner(k) = inner(k)+it_incr
                outer(k) = outer(k)+ot_incr
            endfor
        endfor
    endif
endfor
if (keyword_set(symmetric)) then begin
    inner=2.0*inner
    outer=2.0*outer
endif
total = inner+outer
if (quiet NE 1) then begin
    print, 'Inner disk torque = ', inner
    print, 'Outer disk torque = ', outer
    print, 'Total disk torque = ', total
endif
return
end
