pro v1d, nb=nb, field=array, center=center, overplot=overplot, radix=radix,$
         component=component, dir=dir, noplot=noplot,ccolor=ccolor,$
         velocity=velocity,help=help,abs=abs,$
         levels=levels,xtitle=xtitle,ytitle=ytitle,$
         _extra = e
common HD_param, str_par
;+
; NAME:
;       V1D
;
; PURPOSE:
;       Plots and/or gets a hydrodynamics field of a 1D simulation
;       at some time step. The mesh needs to be 1D, however there
;       may be several non-vanishing velocity components. This
;       procedure automatically finds the size, axis range and name
;       of the plot. It can overplot several curves, and inherits
;       of all the keywords of the 'PLOT' procedure.
;
; CATEGORY:
;       Plotting routine
;
; CALLING SEQUENCE:
;       V1D [, NB=nb] [, FIELD=field] [, CENTER=center]] [, /OVERPLOT]
;       [, RADIX=radix] [, COMPONENT=0,1 or 2] [, DIR=dir] [, /NOPLOT] 
;       [, /VELOCITY], [,LEVELS=levels] [,/CCOLOR]
;       [,XTITLE=xtitle] [,YTITLE=ytitle] [,/ABS]
;
; NB=nb: output number to analyze. Defaults to 0 and is sticky.
;
; FIELD=field: upon return, field is an array that contains the
; (abscissa sorted) HD array.
;
; CENTER=center: upon return, center is a sorted array that contains the
; value of the abscissa of the center of each zone.
; 
; OVERPLOT: uses OPLOT instead of PLOT procedure. 
;
; RADIX: uses the default radix 'density'. Can be changed to
; 'velocity' to display a velocity component
;
; COMPONENT: when the 'velocity' radical is active, component defaults
; to 0. Can be changed to 1 or 2. 
;
; DIR: the dir relative to the current working directory where to
; look for the arrays. Defaults to '.' (current working directory),
; and is sticky.
;
; NOPLOT: does not produce any graphical output. Useful if field and
; center variables are specified, in which case one gets the abscissa
; value for the center of each zone in 'center', and the corresponding
; field value in 'field'
;
; VELOCITY: specifying /VELOCITY is equivalent to RADIX='velocity'
;
; LEVELS=levels: specify the levels to be plotted. Defaults to all
; (i.e. INDGEN(levmax+1)). It can be either an integer or a vector of
; integers.
;
; CCOLOR: represents different CPU outputs with different colors
;
; ABS: plots the field absolute value instead of the field itself.
;
; XTITLE & YTITLE: specify a label along the X (resp. Y) axis
;-

   if (keyword_set(help)) then begin
       doc_library, 'v1d'
       return
   endif
;   my_colors
   radical = 'density'
   comp = 0
   if keyword_set (component) then comp=component
   if keyword_set (radix) then radical=radix
   if keyword_set (velocity) then radical='velocity'
   readstruct,dir=dir,nb=nb
   s=str_par.sgrid
   if n_elements(levels) EQ 0 then levels = indgen(long(s.maxlevel)+1)
   levels=[levels]
   if n_elements(xtitle) EQ 0 then xtitle=s.coordname[0]
   if n_elements(ytitle) EQ 0 then begin
       ytitle="Density"
       if keyword_set(velocity) then ytitle="Velocity"
   endif
   nx=s.nx
   ny=s.ny
   nz=s.nz
   coord_system = s.coordtype
   ngh=s.nghost
   problem1d=0
   for i=0,s.ngrid-1 do begin
       if (nx[i]+ny[i]+nz[i] NE max([nx[i],ny[i],nz[i]]+2)) then problem1d = 1
   endfor
   if problem1d then begin
       printf, -2, 'This does not seem to be a 1D system.'
       printf, -2, 'Cannot produce output for that...'
       return
   endif
   xtot=[-1d123]
   ftot=xtot
   coltot = xtot
   for j=min(levels),max(levels),+1 do begin
       if (([where(levels-j EQ 0)])(0) NE -1) then begin
           lev=[j]
           desclist,level=lev,nb=nb,list=list,dir=dir
           if (list[0] NE -1) then begin
               ncpugrids = (size(list))(1)
               for i=0, ncpugrids-1 do begin
                   read_field,field=f,edge1=x,size=sz,cpugrid=list(i),$
                     /quiet,radix=radical,component=comp,nb=nb
                   f=reform(f)
                   x=reform(x[1:sz[0]]+x[0:sz[0]-1])*.5
                   keep=[where((xtot LT min(x)) OR (xtot GT max(x)))]
                   if (keep[0] NE -1) then begin
                       xtot=xtot(keep)
                       ftot=ftot(keep)
                       coltot=coltot(keep)
                   endif
                   xtot=[xtot,x]
                   ftot=[ftot,f]
                   coltot=[coltot,f*0+list(i)]
               endfor
           endif
       endif
   endfor
   xtot=xtot[1:*]
   ftot=ftot[1:*]
   coltot=coltot[1:*]
   index=sort(xtot)
   xtot=xtot(index)
   ftot=ftot(index)
   coltot=coltot(index)
   center = xtot
   array = ftot
   if keyword_set(abs) then ftot=abs(ftot)
   if keyword_set(noplot) then return
   if NOT(keyword_set(OVERPLOT)) then begin
       plot,xtot,ftot,xtitle=xtitle,ytitle=ytitle,_extra=e
   endif else begin
       oplot,xtot,ftot,_extra=e
   endelse
   if (keyword_set(ccolor)) then begin
       n=(size(xtot))(1)
       for i=0,n-1 do begin
           oplot,[xtot(i)],[ftot(i)],color=3*(coltot(i)+1),_extra=e
       endfor
   endif
return
end
