#include "mhd.h"

#define NBDATA 5

static RadixStruct *radix[NBDATA];

/*
 definition of the initial conditions
 (called by InitialCondition in Init.c)
*/
inline void keplerian_create()
{
  int component;
  for (component=0; component<NBDATA; component++)
    radix[component] = prs_malloc(sizeof(RadixStruct));
  radix[_vrad_]->constant = -1.5*VISCOSITY;
  radix[_vrad_]->radius_slope = -1.0;
  radix[_vrad_]->sintheta_slope = 0.0;
  radix[_vcolatitude_]->constant = 0.0;
  switch (CoordType) {
  case CYLINDRICAL : // 2D
    radix[_density_]->constant = SIGMA0;
    radix[_density_]->radius_slope = SIGMASLOPE;
    radix[_density_]->sintheta_slope = 0.;
    radix[_energy_]->constant = ASPECTRATIO*ASPECTRATIO;
    radix[_energy_]->radius_slope = -1.;
    radix[_energy_]->sintheta_slope = 0.;
    radix[_vazimuth_]->constant = sqrt(1.+(SIGMASLOPE+radix[_energy_]->radius_slope)*ASPECTRATIO*ASPECTRATIO);
    radix[_vazimuth_]->radius_slope = -1.5;
    radix[_vazimuth_]->sintheta_slope = 0.;
    break;
  case SPHERICAL : // 3D
    radix[_density_]->constant = SIGMA0/ASPECTRATIO/sqrt(2.0*M_PI);
    radix[_density_]->radius_slope = SIGMASLOPE-1.;
    radix[_density_]->sintheta_slope = 1./(ASPECTRATIO*ASPECTRATIO);
    radix[_energy_]->constant = \
      (ASPECTRATIO*ASPECTRATIO)/(1.+(-SIGMASLOPE+2.)*ASPECTRATIO*ASPECTRATIO);
    radix[_energy_]->radius_slope = -1.;
    radix[_energy_]->sintheta_slope = 0.;
    radix[_vazimuth_]->constant = 1./sqrt(1.+(-SIGMASLOPE+2.)*ASPECTRATIO*ASPECTRATIO);
    radix[_vazimuth_]->radius_slope = -1.5;
    radix[_vazimuth_]->sintheta_slope = -1.;
    break;
  }
}

/*
  set up of the initial conditions
  (by InitialCondition in Init.c)
*/
inline real keplerian_init(component, radius, colatitude)
     int component;
     real radius, colatitude;
{
  real init;
  if (radix[0] == NULL) {
    prs_stderr ("You try to initialize a Keplerian disk but you do not\n");
    prs_error  ("have set the KEPLERIAN flag in your parameter file");
  }
  init = radix[component]->constant * pow(radius,radix[component]->radius_slope) \
    * pow(sin((colatitude-M_PI/2.0)*1.0+M_PI/2.0),radix[component]->sintheta_slope);
  /* The sine formulation in the above line is to check */
  /* the impact of departure from hydrostatic equilibrium, */
  /* since it is easy to tune the disk thickness by changing */
  /* the 1.0 multiplicative factor. */
  if (component==_vazimuth_) init -= OMEGAFRAME;
  return init;
}

/*
  gravitational external potential
  (for ComputeExternalPotential in ExtPot.c)
*/
inline real keplerian_pot(radius, azimuth, thirdcoord)
     real radius, azimuth, thirdcoord;
{
  real colatitude, z, x;
  real potential, planetmass, MassTaper;
  if (MASSTAPER > 1e-15) {
    MassTaper = GlobalDate/(MASSTAPER*2.0*M_PI);
    MassTaper = (MassTaper > 1.0 ? 1.0 : pow(sin(MassTaper*M_PI/2.0),2.0));
  } else
    MassTaper = 1.0;
  planetmass = PLANETMASS * MassTaper;
  switch (CoordType) {
  case CYLINDRICAL :
    z = thirdcoord;
    potential = -1./sqrt(radius*radius + z*z);
    x = radius*cos(azimuth);
    potential += -planetmass						\
      / (sqrt(1. + radius*radius -2.*radius*cos(azimuth) + z*z + SMOOTHING*SMOOTHING));
    potential += planetmass*x;
    break;
  case SPHERICAL :
    colatitude = thirdcoord;
    potential = -1./radius;
    x = radius*cos(azimuth)*sin(colatitude);
    potential += -planetmass						\
      / sqrt(1. + radius*radius -2.*radius*sin(colatitude)*cos(azimuth) + SMOOTHING*SMOOTHING);
    potential += planetmass*x;
    break;
  }
  return potential;
}

/*
  for a better fill of the ghosts
  (by ExecCommUp in comm_fill.c and ExecComm in comm_exec.c)
*/
inline real keplerian_comm(value, component, radius, colatitude, direction)
     real 	value;
     real 	radius, colatitude;
     int 	direction;
     long       component;
{
  real new_value = value;
  if ( (component==_density_) || (component==_energy_) || (component==_vazimuth_) ) {
    //if (component==_density_){
      if ((direction==+1) && (component==_vazimuth_)) new_value += OMEGAFRAME;
      new_value *= pow(radius,-(direction)*radix[component]->radius_slope) \
	* pow(sin(colatitude),-(direction)*radix[component]->sintheta_slope);
      if ((direction==-1) && (component==_vazimuth_)) new_value -= OMEGAFRAME;
    }
  return new_value;
}

/*
  better boundary conditions
  (for boundary in boundary.c)
*/
real keplerian_boundary(condition, component, value, x, xg)
     int condition, component;
     real value, x, xg;
{
  real ghost;
  ghost = value;
  if (component==_density_ || component==_energy_ || component==_vazimuth_) {
    if (component==_vazimuth_) ghost += OMEGAFRAME;
    switch (condition) {
    case 11 : // radius
      ghost *= pow(xg/x,radix[component]->radius_slope);
      break;
    case 12 : // colatitude
            if (component == _density_)
	      ghost *= pow(sin(xg)/sin(x),radix[component]->sintheta_slope);
      break;
    }
    if (component==_vazimuth_) ghost -= OMEGAFRAME;
  } else if ((component==_vrad_) && (condition == 11)) {
    ghost *= pow(xg/x,-SIGMASLOPE-1.);
  }
  return ghost;
}
